﻿If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessages' ))
  If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_UserMessages' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'IsConf' )
	Alter Table dbo.A_UserMessages
	Add [IsConf] int default 0
GO

Update dbo.A_UserMessages 
Set IsConf = 0
GO

if not exists ( select * from sysobjects where id = object_id ( 'dbo.A_UserMessageFiles' ))
	Create table dbo.A_UserMessageFiles
	(
		id uniqueidentifier primary key,
		sentdate datetime not null,
		idmessage uniqueidentifier not null,
		filepath nvarchar(4000),
		filesize int
	)
GO


if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_Add' ))
	Drop procedure dbo.A_UserMessages_Add
GO

Create Procedure [dbo].A_UserMessages_Add
	@id		uniqueidentifier,
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@sentdate	datetime,
	@message	text,
	@isconf		int = 0
as
begin
	Declare @isviewed bit, @vieweddate datetime
	if ( @isconf > 0 )
 	  Select @isviewed = 1, @vieweddate = @sentdate
	else
	  Select @isviewed = 0, @vieweddate = null

	Insert into A_UserMessages ( id, idusera, iduserb, sentdate, message, viewed, vieweddate, isconf )
	Values ( @id, @idusera, @iduserb, @sentdate, @message, @isviewed, @vieweddate, @isconf )
end
GO


if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_GetHistory' ))
	Drop procedure dbo.A_UserMessages_GetHistory
GO

Create procedure dbo.A_UserMessages_GetHistory
	@dtstart DateTime,
	@dtstop  DateTime,
	@searchtext nvarchar ( 100 ),
	@iduser uniqueidentifier,
	@idopponent uniqueidentifier
as
begin
	declare @ids table ( Id uniqueidentifier )

	if (( @idopponent != '00000000-0000-0000-0000-000000000000' ) and ( @searchtext != '' ))

		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IduserB = @idopponent) 
					or (IdUserB = @iduser and IdUserA = @idopponent))
--					or (IsConf = 1 and IdUserB = @idopponent
				
				and Message like '%' + @searchtext + '%'

	else if ( @idopponent != '00000000-0000-0000-0000-000000000000' )

		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IduserB = @idopponent) or (IdUserB = @iduser and IdUserA = @idopponent))

	else if ( @searchtext != '' )

		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IduserB = @iduser)
				and Message like '%' + @searchtext + '%'

	else

		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IduserB = @iduser)			
	


	-- Первая выборка - сами сообщения
	Select msg.Id, IdUserA, IsNull ( u1.Name, IsNull ( c1.Name, '???' )) as UserAName, IdUserB, IsNull ( u2.Name, IsNull ( c2.Name, '???' )) as UserBName, SentDate, Message, Viewed, ViewedDate, IsConf
	From A_UserMessages msg
		Inner join @ids ids on msg.Id = ids.Id
		Left join A_Users u1 on msg.IdUserA = u1.Id
		Left join A_Users u2 on msg.IdUserB = u2.Id
		Left join A_Stat_Conference c1 on ( msg.IsConf = 1 and msg.IdUserA = c1.Id )
		Left join A_Stat_Conference c2 on ( msg.IsConf = 1 and msg.IdUserB = c2.Id )
	Order by SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, umf.IdMessage, umf.FilePath, umf.FileSize
	From A_UserMessages msg
		Inner join @ids ids on msg.Id = ids.Id
		Inner join A_UserMessageFiles umf on umf.IdMessage = msg.Id and umf.SentDate = msg.SentDate
	Order by umf.SentDate
end
GO


if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_GetLastForUser' ))
	Drop procedure dbo.A_UserMessages_GetLastForUser
GO

Create Procedure [dbo].A_UserMessages_GetLastForUser
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@count		int
as
begin
	Declare @tmp table ( 
		Inc	int identity ( 1, 1 ),
		Id	uniqueidentifier,
		IdUserA uniqueidentifier,
		IdUserB	uniqueidentifier,
		SentDate DateTime,
		Message text collate database_default,
		Viewed bit,
		ViewedDate DateTime,
		IsConf int )

	Insert into @tmp ( Id, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate, IsConf )
		Select Top 100 *
		FROM A_UserMessages 
		Where ( IdUserA = @idusera And IdUserB = @iduserb )
			or ( IdUserA = @iduserb And IdUserB = @idusera )
		Order By SentDate Desc


	-- Первая выборка - сами сообщения
	Select tmp.Id, IdUserA, IsNull ( u1.Name, IsNull ( c1.Name, '???' )) as UserAName, IdUserB, IsNull ( u2.Name, IsNull ( c2.Name, '???' )) as UserBName, SentDate, Message, Viewed, ViewedDate, IsConf
	FROM @tmp tmp
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
		Left join A_Stat_Conference c1 on IdUserA = c1.Id
		Left join A_Stat_Conference c2 on IdUserB = c2.Id
	Where Inc <= @count
	Order by SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, umf.IdMessage, umf.FilePath, umf.FileSize
	From @tmp msg
		Inner join A_UserMessageFiles umf on umf.IdMessage = msg.Id and umf.SentDate = msg.SentDate
	Order by umf.SentDate
end
GO



if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_GetNew' ))
	Drop procedure dbo.A_UserMessages_GetNew
GO

Create Procedure [dbo].A_UserMessages_GetNew
	@iduser uniqueidentifier
as
begin
	Declare @allusers bit
	Set @allusers = 0
	if ( @iduser = '00000000-0000-0000-0000-000000000000' )
	   Set @allusers = 1	

	-- Не нужны сообщения из чатов конференций, поскольку они сразу считаются прочитанными

	Declare @t table (Id uniqueidentifier, SentDate datetime)
	Insert into @t (Id, SentDate)
		Select Id, SentDate
		From A_UserMessages msg
		Where (( IdUserB = @iduser ) or ( @allusers = 1 ))
			and ( Viewed = 0 ) and ( IsConf = 0 )

	-- Первая выборка - сами сообщения
	Select msg.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, msg.SentDate, Message, Viewed, ViewedDate, IsConf 
	From A_UserMessages msg
		Inner join @t t on msg.Id = t.Id
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Order by msg.SentDate

	
	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, umf.IdMessage, umf.FilePath, umf.FileSize
	From @t msg
		Inner join A_UserMessageFiles umf on umf.IdMessage = msg.Id and umf.SentDate = msg.SentDate
	Order by umf.SentDate
end
GO



if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_GetNewForActive' ))
	Drop procedure dbo.A_UserMessages_GetNewForActive
GO

Create Procedure [dbo].A_UserMessages_GetNewForActive
	@makeviewed bit = 1
as
begin	
	-- Запрос для обеспечения периодического опроса БД на предмет появления внешним образом новых сообщений

	-- Достаем активных подключенных
	Declare @active table (id uniqueidentifier)

	-- По A_UserStateHistory. Тут дефолтные пользователи тоже попадают
	/*
	Insert into @active (id)
		Select id--, name
		From ( Select u.id, --u.Name, 
				IsNull (( Select Top 1 h.state as state
					From dbo.A_UserStateHistory h 
					Where h.UserId = u.id
					Order By h.TimeChange Desc ), 0) as state
			From dbo.A_Users u
			Group by u.id--, u.Name
			) a
		Where state > 0
		--Order by name
	*/

	-- По A_CallCenter_UserStateHistory. Тут без дефолтных пользователей и запрос проще
	Insert into @active (id)
		Select OperatorId
		From A_CallCenter_UserStateHistory h
			Inner join A_Users u on u.id = h.operatorid
		Where TimeLeave is null
		Group by OperatorId


	-- Не нужны сообщения из чатов конференций, поскольку они сразу считаются прочитанными

	Declare @t table (Id uniqueidentifier, SentDate datetime)
	Insert into @t (Id, SentDate)
		Select Id, SentDate
		From A_UserMessages msg
		Where (( Viewed = 0 ) and ( IsConf = 0 )
			and (IdUserB in ( Select id from @active )))


	-- Первая выборка - сами сообщения
	Select msg.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, msg.SentDate, Message, Viewed, ViewedDate, IsConf
	From A_UserMessages msg
		Inner join @t t on msg.Id = t.Id
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Order by msg.SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, umf.IdMessage, umf.FilePath, umf.FileSize
	From @t msg
		Inner join A_UserMessageFiles umf on umf.IdMessage = msg.Id and umf.SentDate = msg.SentDate
	Order by umf.SentDate


	-- Считаем их просмотренными ?
	if ( @makeviewed = 1 ) begin

		Update m
		Set m.Viewed = 1
		From A_UserMessages m, @active u
		Where m.IdUserB = u.id and m.Viewed = 0	
	end
end
GO


if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessages_SetViewed' ))
	Drop procedure dbo.A_UserMessages_SetViewed
GO

Create procedure [dbo].[A_UserMessages_SetViewed]
	@idmessage uniqueidentifier
as
begin
	declare @ida uniqueidentifier
	declare @idb uniqueidentifier
	declare @dt datetime

	Select @ida = IdUserA, @idb = IdUserB, @dt = SentDate
	From A_UserMessages 
	Where Id = @idmessage

	if ( @@rowcount = 0 )
		return


	declare @rowcount int
	
	Update A_UserMessages
	Set Viewed = 1, ViewedDate = GetDate ( )
	Where IdUserA = @ida and IdUserB = @idb and SentDate <= @dt and Viewed != 1

	select @rowcount = @@rowcount
	
	Update W_Messages_Dialogs 
	set hasnew = 0 where IdUserB = @ida and IdUserA = @idb and hasnew != 0
	
	Select 120318, @rowcount as updatedcount
end
GO


if exists ( select * FROM sysobjects where id = object_id ( 'dbo.W_UpdateDialogTrigger' ))
	Drop TRIGGER dbo.W_UpdateDialogTrigger
GO

Create TRIGGER [dbo].[W_UpdateDialogTrigger] 
   ON [A_UserMessages]
   AFTER INSERT
AS 
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;
	
	declare @message nvarchar(4000), @count int
	--@iduserA uniqueidentifier,
	--@iduserB uniqueidentifier,
	--@sentdate datetime, 
	
	
	--select 	@iduserA = iduserA,
	--@iduserB = iduserB,
	--@sentdate = sentdate
	--from inserted
	Select count ( * ) 
	From inserted i
	Where i.IsConf = 0

	if ( @@rowcount = 0 )
  	  return;
	
	select @message = substring (um.message, 0, 255)
	from A_UserMessages um, inserted i
	where um.iduserA = i.iduserA
		and um.iduserB = i.iduserB
		and um.sentdate = i.sentdate
	
	update W_Messages_Dialogs 
	set sentdate = i.sentdate, 
		lastmessage = @message,
		messagecount = md.messagecount+1,
		hasnew = 0
	from inserted i, 
		W_Messages_Dialogs md
	where (md.iduserA = i.iduserA and md.iduserB = i.iduserB)
	
	update W_Messages_Dialogs 
	set sentdate = i.sentdate, 
		lastmessage = @message,
		messagecount = md.messagecount+1,
		hasnew = 1
	from inserted i, 
		W_Messages_Dialogs md
	where (md.iduserB = i.iduserA and md.iduserA = i.iduserB)
	
	if ( @@rowcount = 0 ) begin
	
		select @count = count (*)
		from A_UserMessages um,  inserted i
		where (um.iduserA = i.iduserA and um.iduserB = i.iduserB) or (um.iduserB = i.iduserA and um.iduserA = i.iduserB)
		
		insert into W_Messages_Dialogs (iduserA, iduserB, sentdate, messagecount, lastmessage,hasnew)--
		  select iduserA,
			iduserB,
			sentdate,
			@count,
			@message,
			0
		from inserted
		
		insert into W_Messages_Dialogs (iduserB, iduserA, sentdate, messagecount, lastmessage,hasnew)--
		  select iduserA,
			iduserB,
			sentdate,
			@count,
			@message,
			1
		from inserted
	end

END
GO



if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessageFiles_Add' ))
	Drop procedure dbo.A_UserMessageFiles_Add
GO

Create procedure dbo.A_UserMessageFiles_Add
	@id uniqueidentifier,
	@sentdate datetime,
	@idmessage uniqueidentifier,
	@filepath nvarchar(4000),
	@filesize int
as	
begin
	Insert into A_UserMessageFiles (id, sentdate, idmessage, filepath, filesize )
	Values ( @id, @sentdate, @idmessage, @filepath, @filesize )
end
GO



if exists ( select * FROM sysobjects where id = object_id ( 'dbo.A_UserMessageFiles_Remove' ))
	Drop procedure dbo.A_UserMessageFiles_Remove
GO

Create procedure dbo.A_UserMessageFiles_Remove
	@id uniqueidentifier
as	
begin
	Delete From A_UserMessageFiles
	Where id = @id
end
GO

