﻿---------------------------------------------------------

if exists ( select * From sysobjects where id= object_id ( 'dbo.A_RegistrarSession' ))
	Drop table dbo.A_RegistrarSession
GO

Create table dbo.A_RegistrarSession (
	Id int identity( 1, 1 ) Primary key,
	CallId nvarchar(200),
	Account nvarchar(100),
	RecvAddr nvarchar(50),
	IfaceAddr nvarchar(50),
	OverNAT nvarchar(10),
	VIA nvarchar(200),
	CSeq nvarchar(10),
	Nonce nvarchar(100),
	NC nvarchar(50),
	StartTimeX nvarchar(50),
	LocalTimeCreate datetime,
	LocalTimeUpdate datetime	
)
GO

---------------------------------------------------------

if exists ( select * from sysobjects where id= object_id ( 'dbo.A_RegistrarContact' ))
	Drop table dbo.A_RegistrarContact
GO

Create table dbo.A_RegistrarContact (
	Id int identity(1,1) primary key,
	SessionId int,
	DisplayName nvarchar(200),
	URI nvarchar(200),
	Type nvarchar(50),
	Expires int,
	LocalTimeExpire datetime
)
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_RegistrarSession_CallId' and id = object_id(N'[dbo].[A_RegistrarSession]'))
	Drop index [dbo].[A_RegistrarSession].[idx_RegistrarSession_CallId]
GO

if exists (select * from dbo.sysindexes where name = N'idx_RegistrarSession_CallId_Acc' and id = object_id(N'[dbo].[A_RegistrarSession]'))
	Drop index [dbo].[A_RegistrarSession].[idx_RegistrarSession_CallId_Acc]
GO

Create Nonclustered Index [idx_RegistrarSession_CallId_Acc] 
On [dbo].[A_RegistrarSession]([CallId], [Account]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_A_RegistrarContact_SessionId' and id = object_id(N'[dbo].[A_RegistrarContact]'))
	Drop index [dbo].[A_RegistrarContact].[idx_A_RegistrarContact_SessionId]
GO

Create Nonclustered Index [idx_A_RegistrarContact_SessionId] 
On [dbo].[A_RegistrarContact]([SessionId]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_A_RegistrarContact_TimeFilter' and id = object_id(N'[dbo].[A_RegistrarContact]'))
	Drop index [dbo].[A_RegistrarContact].[idx_A_RegistrarContact_TimeFilter]
GO

Create Nonclustered Index [idx_A_RegistrarContact_TimeFilter] 
On [dbo].[A_RegistrarContact]([LocalTimeExpire]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------
---------------------------------------------------------
---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_FilterSessions'))
	Drop procedure dbo.A_Registrar_FilterSessions
GO

Create procedure dbo.A_Registrar_FilterSessions
as
begin
	Declare @dtnow datetime
	Set @dtnow = GetDate()

	Delete
	From A_RegistrarContact
	Where LocalTimeExpire < @dtnow

	Delete
	From A_RegistrarSession
	Where not exists ( Select * From A_RegistrarContact c Where c.SessionId = A_RegistrarSession.Id )
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_DropSession'))
	Drop procedure dbo.A_Registrar_DropSession
GO

Create procedure dbo.A_Registrar_DropSession
	@sessionid int = -1,
	@account nvarchar(100) = '',
	@callid nvarchar(200) = ''	
as
begin
	
	Declare @id int		
	Set @id = @sessionid

	if ( @id <= 0 ) begin	
	    Select @id = Id 
	    From A_RegistrarSession
	    Where ( CallId like @callid )
			and ( Account like @account )
	end

	Delete 
	From A_RegistrarSession
	Where Id = @id

	Delete 
	From A_RegistrarContact
	Where SessionId = @id

end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_UpdateSession'))
	Drop procedure dbo.A_Registrar_UpdateSession
GO

Create procedure dbo.A_Registrar_UpdateSession
	@sessionid int = -1,
	@callid nvarchar(200),
	@account nvarchar(100),
	@recvaddr nvarchar(50),
	@ifaceaddr nvarchar(50),
	@overnat nvarchar(10),
	@via nvarchar(200),
	@cseq nvarchar(10) = '0',
	@nonce nvarchar(100),
	@nc nvarchar(50),
	@starttimex nvarchar(50)
as
begin
	Declare @dtnow Datetime
	Set @dtnow = GetDate()
	
	if ( @sessionid <= 0 ) begin

		Select @sessionid = Id
		From A_RegistrarSession
		Where ( CallId = @callid )
			and ( Account = @account )

		if ( @sessionid is NULL )
			Set @sessionid = -1
	end

	if ( @sessionid > 0 ) begin

	    Update A_RegistrarSession
	    Set RecvAddr = @recvaddr,
			IfaceAddr = @ifaceaddr,
			OverNat = @overnat,
			VIA = @via,
			CSeq = @cseq,
			Nonce = @nonce,
			NC = @nc,
			StartTimeX = @starttimex,
			LocalTimeUpdate = @dtnow	
	    Where Id = @sessionid

		if ( @@rowcount > 0 ) begin
			Select cast ( @sessionid as int )
			return
		end
	
	end


	Insert into A_RegistrarSession ( CallId, Account, RecvAddr, IfaceAddr, OverNAT, VIA, CSeq, Nonce, NC, StartTimeX, LocalTimeCreate, LocalTimeUpdate )
	Values ( @callid, @account, @recvaddr, @ifaceaddr, @overnat, @via, @cseq, @nonce, @nc, @starttimex, @dtnow, @dtnow )

	Select cast ( @@identity as int )
	
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_UpdateContact'))
	Drop procedure dbo.A_Registrar_UpdateContact
GO

Create procedure dbo.A_Registrar_UpdateContact
	@sessionid int,
	@displayname nvarchar(200) = '',
	@uri nvarchar(200),
	@type nvarchar(50),
	@expires int	
as
begin
		
	Declare @dtnow datetime, @localtimeexpire datetime
	Set @dtnow = GetDate()
	Set @localtimeexpire = DateAdd ( ss, @expires + 5, @dtnow )

	Update A_RegistrarContact
	Set Expires = @expires,
		LocalTimeExpire = @localtimeexpire
	Where SessionId = @sessionid
		And URI like @uri

	if ( @@rowcount = 0 ) begin

		Insert into A_RegistrarContact ( SessionId, DisplayName, URI, Type, Expires, LocalTimeExpire )
		Values ( @sessionid, @displayname, @uri, @type, @expires, @localtimeexpire )
	end
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_GetData'))
	Drop procedure dbo.A_Registrar_GetData
GO

Create procedure dbo.A_Registrar_GetData
as
begin
	-- Сначала лишние отфильтруем
	exec dbo.A_Registrar_FilterSessions

	-- Потом оставшиеся вернем
	Select CallId, Account, RecvAddr, IfaceAddr, OverNAT, VIA, CSeq, Nonce, NC, StartTimeX,
		DisplayName, URI, Type, Expires
	From A_RegistrarSession s
		Inner join A_RegistrarContact c on c.SessionId = s.Id
	Where c.LocalTimeExpire > GetDate()
	Order by Account, CallId
end
GO

---------------------------------------------------------
---------------------------------------------------------
---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.[A_Timer_MinuteProc]'))
	Drop procedure dbo.[A_Timer_MinuteProc]
GO

Create  Procedure [dbo].[A_Timer_MinuteProc]
	@dtnow datetime = null
as
begin
	if ( @dtnow is null )	
		set @dtnow = GetDate()
--------------------------------------------------------------------------------------------
-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
--------------------------------------------------------------------------------------------
   	Begin tran DbProcActive

		Declare @dtactive datetime
		Select @dtactive = valuedatetime from A_Settings where name like 'DB_procactive'
		if ( not @dtactive is null )
 		  if ( DateDiff ( mi, @dtactive, @dtnow ) < 10 ) begin
		    Rollback tran DbProcActive
		    return
		  end

		Update A_Settings Set valuedatetime = @dtnow where name like 'DB_procactive'
		if ( @@rowcount = 0 )
		  Insert into A_Settings (name, valuedatetime) Values ( 'DB_procactive', @dtnow )

   	Commit tran DbProcActive
--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------

	-- Очистка неактуальных регистрационных сессий
	exec dbo.A_Registrar_FilterSessions

	--Время по которое производить расчет намеренно назад на 30 секунд отводим
	-- чтобы исключить возможность попадания в интервалы, когда отложенные сохранения в БД идут
	-- и когда одна таблица уже заполнена, а другая еще нет данными до текущего момента.
	-- ну и еще случаи частичного заполнения строки, когда данные вставляются в момент выборки
	Declare @tilldt DateTime
	Set @tilldt = DateAdd ( ss, -30, @dtnow )

	exec A_CallCenter_Temp_UpdateOperatorStates @tilldt

--------------------------------------------------------------------------------------------
-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
--------------------------------------------------------------------------------------------
	Delete From A_Settings where name like 'DB_procactive'
--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------
end
GO

-----------------------------

--exec A_Registrar_UpdateSession @sessionid = -1, @callid = 'callid', @account = 'account', @recvaddr = 'recvaddr', @via = 'via', @nonce = 'nonce', @nc = 'nc', @starttimex = 'starttimex'
--exec A_Registrar_UpdateContact @sessionid = 10, @uri = 'uri', @type = 'type', @expires = 30
--exec A_Registrar_UpdateSession @sessionid = 9, @callid = 'callid', @account = 'account', @recvaddr = 'recvaddr', @via = 'via', @nonce = 'nonce', @nc = 'nc', @starttimex = 'starttimex'
--exec A_Registrar_UpdateContact @sessionid = 9, @uri = 'uri', @type = 'type', @expires = 30
--exec A_Registrar_FilterSessions
--exec A_Registrar_GetData
--Select * From A_RegistrarSession s Inner join A_RegistrarContact c on c.SessionId = s.Id
--exec A_Registrar_GetData
--Select * FROM A_RegistrarSession
--Select * FROM A_RegistrarContact


--Truncate table A_RegistrarSession
--Truncate table A_RegistrarContact
