﻿if exists ( Select * from sysobjects where id=object_id ( 'dbo.A_UserRoles_Load_Roles' ))
	Drop procedure dbo.A_UserRoles_Load_Roles
GO

Create Procedure [dbo].[A_UserRoles_Load_Roles]
	@onlyforuserid uniqueidentifier = null,
	@onlyforroleid uniqueidentifier = null,
	@skipdominant bit = 0
As
Begin

    if ( @onlyforuserid is null ) begin

		Select Id, Name, BaseRole, Domination, UserCount, 0 as levelNesting 
		From dbo.A_UserRoles_Roles a
			left join ( Select IdRole, UserCount = Count ( * )
						From A_UserRoles_RolesOfUsers
						Group By IdRole
			) b On a.Id = b.IdRole
		Where (( @skipdominant = 0 ) or ( Domination != 1 ))
			And ((@onlyforroleid is null) or ( a.Id = @onlyforroleid ))
		Order by Name

    end else begin

		Declare @roles table (id uniqueidentifier, levelNesting int)
		Insert into @roles ( id, levelNesting )
			Select IdRole, 0
			From A_UserRoles_RolesOfUsers
			Where IdUser = @onlyforuserid

		Declare @i int
		Set @i = 0			
		while ( @@rowcount > 0 ) begin
			Set @i = @i + 1
			if ( @i > 20 )
				break
				
			Insert into @roles ( id, levelNesting )
				Select rx.Id, @i
				From A_UserRoles_Roles rs
					Inner join @roles r on r.Id = rs.Id
					Inner join A_UserRoles_Roles rx on rx.Id = rs.BaseRole
				Where (r.levelNesting = @i - 1)
					and not exists ( Select id from @roles x where x.Id = rx.Id )
		end		

		Select a.Id, a.Name, a.BaseRole, a.Domination, b.UserCount, r.levelNesting 
		From dbo.A_UserRoles_Roles a
			inner join @roles r on a.Id = r.id
	    	left join ( Select IdRole, UserCount = Count ( * )
						From A_UserRoles_RolesOfUsers
						Group By IdRole 
			) b On a.Id = b.IdRole
		Where (( @skipdominant = 0 ) or ( a.Domination != 1 ))
			And ((@onlyforroleid is null) or ( a.Id = @onlyforroleid ))
		Order by Name

    end

End
GO


if exists ( select * from sysobjects where id=object_id ( 'dbo.A_UserRoles_Load_RightsOfRoles' ))
	Drop procedure dbo.A_UserRoles_Load_RightsOfRoles
GO

Create Procedure [dbo].[A_UserRoles_Load_RightsOfRoles]
	@onlyforuserid uniqueidentifier = null,
	@onlyforroleid uniqueidentifier = null,
	@skipdominant bit = 0
As
Begin
	if ( @onlyforuserid is null ) begin
	
		Select a.IdRole, b.Code, a.Value, 0 as levelNesting
		From dbo.A_UserRoles_RightsOfRoles a
			inner join A_UserRoles_Roles roles on roles.Id = a.IdRole
			left join A_UserRoles_Rights b On a.IdRight = b.id
		Where ((@skipdominant = 0) or (roles.Domination = 0))
			And ((@onlyforroleid is null) or ( roles.Id = @onlyforroleid ))

	end else begin

		Declare @roles table (id uniqueidentifier, levelNesting int)
		Insert into @roles ( id, levelNesting )
			Select IdRole, 0
			From A_UserRoles_RolesOfUsers
			Where IdUser = @onlyforuserid

		Declare @i int
		Set @i = 0			
		while ( @@rowcount > 0 ) begin
			Set @i = @i + 1
			if ( @i > 20 )
				break
				
			Insert into @roles ( id, levelNesting )
				Select rx.Id, @i
				From A_UserRoles_Roles rs
					Inner join @roles r on r.Id = rs.Id
					Inner join A_UserRoles_Roles rx on rx.Id = rs.BaseRole
				Where (r.levelNesting = @i - 1)
					and not exists ( Select id from @roles x where x.Id = rx.Id )
		end

		Select a.IdRole, b.Code, a.Value, r.levelNesting 
		From dbo.A_UserRoles_RightsOfRoles a
			inner join @roles r on r.id = a.IdRole
			inner join A_UserRoles_Roles roles on roles.Id = a.IdRole
			left join A_UserRoles_Rights b On a.IdRight = b.id
		Where ((@skipdominant = 0) or (roles.Domination = 0))
			And ((@onlyforroleid is null) or ( roles.Id = @onlyforroleid ))
		
	end

End
GO



if exists ( select * from sysobjects where id=object_id ( 'dbo.A_UserRoles_Load_RolesOfUsers' ))
	Drop procedure dbo.A_UserRoles_Load_RolesOfUsers
GO

Create Procedure [dbo].[A_UserRoles_Load_RolesOfUsers]
	@onlyforuserid uniqueidentifier = null,
	@onlyforroleid uniqueidentifier = null,
	@skipdominant bit = 0
As
Begin
	if ( @onlyforuserid is null ) begin
		
		Select IdUser, IdRole, b.Domination, 0 as levelNesting
		From dbo.A_UserRoles_RolesOfUsers a
			left join A_UserRoles_Roles b On a.IdRole = b.Id
		Where ( b.Domination is not null )
			And ((@skipdominant = 0) or (b.Domination = 0))
			And ((@onlyforroleid is null) or ( b.Id = @onlyforroleid ))

	end else begin

		Declare @roles table (id uniqueidentifier, levelNesting int)
		Insert into @roles ( id, levelNesting )
			Select IdRole, 0
			From A_UserRoles_RolesOfUsers
			Where IdUser = @onlyforuserid

		Declare @i int
		Set @i = 0			
		while ( @@rowcount > 0 ) begin
			Set @i = @i + 1
			if ( @i > 20 )
				break
				
			Insert into @roles ( id, levelNesting )
				Select rx.Id, @i
				From A_UserRoles_Roles rs
					Inner join @roles r on r.Id = rs.Id
					Inner join A_UserRoles_Roles rx on rx.Id = rs.BaseRole
				Where (r.levelNesting = @i - 1)
					and not exists ( Select id from @roles x where x.Id = rx.Id )
		end

		Select a.IdUser, a.IdRole, b.Domination, r.levelNesting 
		From dbo.A_UserRoles_RolesOfUsers a
			inner join @roles r on r.Id = a.IdRole
			left join A_UserRoles_Roles b On a.IdRole = b.Id
		Where (b.Domination is not null)
			And ((@skipdominant = 0) or (b.Domination = 0))
			And ((@onlyforroleid is null) or ( b.Id = @onlyforroleid ))
	end

End
GO