﻿If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Chat_MessageFile' ))
        If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Chat_MessageFile'
                                                        and TABLE_SCHEMA like 'dbo'
                                                                and COLUMN_NAME like 'FileName' )
        Alter Table dbo.A_Chat_MessageFile
        Add [FileName] nvarchar(200)
GO


Update dbo.A_Chat_MessageFile
Set [FileName] = case when ( [FilePath] is NULL ) 
					then ''
					else
						case when ( PatIndex ( '%\%', [FilePath] ) > 0 )
							then Reverse( Substring( Reverse( [FilePath] ), 1, PatIndex( '%\%', Reverse( [FilePath] ) ) - 1)) 
							else [FilePath]  
						end
					end
Where [FileName] is NULL
GO

------------------------------------------------------------------
if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Chat_Message_AddFile' ))
	Drop procedure dbo.[A_Chat_Message_AddFile]
GO

Create procedure [dbo].[A_Chat_Message_AddFile]
	@dialogid	uniqueidentifier,
	@messageid	uniqueidentifier,
	@fileid		uniqueidentifier,
	@filepath	nvarchar(4000),
	@filename	nvarchar(200) = '',
	@filesize	int = NULL
as
begin

	-- Добавляет файл к сообщению	

	Insert into dbo.[A_Chat_MessageFile] ( [Id], [IdMessage], [FilePath], [FileSize], [FileName] )
	Values ( @fileid, @messageid, @filepath, @filesize, @filename )

end
GO

------------------------------------------------------------------
if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Chat_Messages_GetAllNewX' ))
	Drop procedure dbo.A_Chat_Messages_GetAllNewX
GO

Create procedure [dbo].[A_Chat_Messages_GetAllNewX]
	@authorizeduserid	uniqueidentifier
as
begin
	-- Возвращает количествно непрочитанных пользователем сообщений всего
	
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#msg' ))
		Drop table #msg

	Create table #msg ( IdMessage uniqueidentifier )

	-- Фильтр
	Insert into #msg ( IdMessage )
		Select Top 1000 
			m.IdMessage
		From dbo.A_Chat_NewMessages nm
			Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		Where nm.IdUser = @authorizeduserid
		Order by m.DateUTC Desc


	-- Сообщения
	Select Top 1000 
		m.IdDialog,  
		m.IdMessage, 
		m.IdUser as AuthorId, 
		u.Login as AuthorLogin, 
		u.Name as AuthorName, 
		m.DateUTC,
		m.Text
	From #msg filter
		Inner join dbo.A_Chat_Messages m on m.IdMessage = filter.IdMessage
		Left join dbo.A_Users u on u.Id = m.IdUser	
	Order by m.DateUTC Desc


	-- Файлы привязанные
	Select	[IdMessage],
			[Id],
			[FilePath],
			[FileSize],
			[FileName]
	From dbo.[A_Chat_MessageFile]
	Where IdMessage in (select IdMessage from #msg)
	Order by IdMessage

end
GO

------------------------------------------------------------------
if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Chat_Messages_GetTopX' ))
	Drop procedure dbo.[A_Chat_Messages_GetTopX]
GO

Create procedure [dbo].[A_Chat_Messages_GetTopX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя

		-- Является ли участником чата
		Declare @ismember int
		
		Select @ismember = Count(*)
		From dbo.A_Chat_Members
		Where IdDialog = @dialogid
			And IdUser = @authorizeduserid

		if ( @ismember = 0 )
			return

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		-- Фильтр для текста
		Declare @filtertext nvarchar(4000)
		Set @filtertext = ''	

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @filtertext = ''

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @filtertext = ' and (Text like ''%' + @search + '%'')'

		end else begin

			Set @filtertext = ' and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))'

		end
		

		Declare @sql nvarchar(4000)

		-- Сначала ограничение (на последней странице меньше записей). 
		Declare @rowcount int			
		Set @sql = 'Select @rc = Count(*) 
					From dbo.[A_Chat_Messages]
					Where (IdDialog = @dlg)' + @filterdate + @filtertext
		exec sp_executesql @sql, N'@rc int OUTPUT, @dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', 
					@rc = @rowcount OUTPUT, @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		-- Корректировка @count 
		if ( @rowcount < @start + @count )
			Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

		-- Потом выборка идентификаторов
		if ( @count > 0 ) begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2
			
				Insert into #interval ( IdMessage )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + @filtertext + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end

				
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages]
											where IdMessage = i.IdMessage 
												and IdDialog = @dialogid
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members]
											where IdDialog = @dialogid
												and IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = @dialogid 
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Desc
		
		-- Файлы привязанные
		Select	[IdMessage],
				[Id],
				[FilePath],
				[FileSize],
				[FileName]
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO

------------------------------------------------------------------
if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Chat_Messages_GetTopXByOldMapper' ))
	Drop procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
GO

Create procedure [dbo].[A_Chat_Messages_GetTopXByOldMapper]
	@authorizeduserid	uniqueidentifier,
	@opponentid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя
	

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier, IdDialog uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		-- Фильтр для текста
		Declare @filtertext nvarchar(4000)
		Set @filtertext = ''	

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @filtertext = ''

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @filtertext = ' and (Text like ''%' + @search + '%'')'

		end else begin

			Set @filtertext = ' and ((Text like ''%' + @search1 + '%'') or (Text like ''%' + @search2 + '%''))'

		end


		Declare @sql nvarchar(4000)

		-- Сначала ограничение (на последней странице меньше записей). 
		Declare @rowcount int			
		Set @sql = 'Select @rc = Count(*) 
					From dbo.[A_Chat_Messages] msg
						Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) 
							and (   ((mem.IdUser = @me) and (msg.IdUser = @author)) 
								 or ((mem.IdUser = @author) and (msg.IdUser = @me)))
					Where (1=1)' + @filterdate + @filtertext
		exec sp_executesql @sql, N'@rc int output, @author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@rc = @rowcount OUTPUT, @author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		-- Корректировка @count 
		if ( @rowcount < @start + @count )
			Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

		-- Потом выборка идентификаторов
		if ( @count > 0 ) begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) 
									and (   ((mem.IdUser = @me) and (msg.IdUser = @author)) 
										 or ((mem.IdUser = @author) and (msg.IdUser = @me)))
							Where (1=1)' + @filterdate + @filtertext + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end 
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages] nm
											where nm.IdMessage = i.IdMessage 
												and nm.IdDialog = i.IdDialog
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members] mem
											where mem.IdDialog = i.IdDialog
												and mem.IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = i.IdDialog
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Asc
		
		-- Файлы привязанные
		Select	[IdMessage],
				[Id],
				[FilePath],
				[FileSize],
				[FileName]
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO