﻿-------------------------------------------------------------------------------
-------------   Добавление полей в таблицу WO_Dashboards_Widgets   ------------
-------------------------------------------------------------------------------
If exists ( Select * From oktell_settings.dbo.sysobjects where id = object_id ( 'oktell_settings.dbo.WO_Dashboards_Widgets' ))
  If not Exists ( Select * From oktell_settings.INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'WO_Dashboards_Widgets' 
								and TABLE_CATALOG like 'oktell_settings'
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME in ('IdConstructorFields') )
	Alter Table [oktell_settings].[dbo].[WO_Dashboards_Widgets]
	Add
		[IdConstructorFields] uniqueidentifier default NULL,
		[IdConstructorFilters] uniqueidentifier default NULL,
		[IdConstructorAxis] int default NULL
GO


Update [oktell_settings].[dbo].[WO_Dashboards_Widgets] 
Set [IdConstructorFields] = null, [IdConstructorFilters] = null, [IdConstructorAxis] = null
GO

---------------------------
-- WO_Dashboards_Widgets --
---------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[WO_Dashboards_Widgets]', 'V'))
DROP VIEW [dbo].[WO_Dashboards_Widgets]
GO

CREATE VIEW [dbo].[WO_Dashboards_Widgets]
AS
	SELECT	*
	FROM [oktell_settings].[dbo].[WO_Dashboards_Widgets]
GO

--------------------------------------
-----     Изменение процедур     -----
--------------------------------------

------------------------------------
-- WO_Dashboards_Dashboard_Create --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Create]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Create]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Create]
	@authorizeduserid	uniqueidentifier,
	@dashboardname	nvarchar(100),
	@projectid	nvarchar(50)
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			declare	@dashboardid uniqueidentifier
			set	@dashboardid = newid()
			
			if ( @projectid like '' )
				Set @projectid = NULL
			
			-- Добавление нового дашборда
			insert into [dbo].[WO_Dashboards_Dashboards]
				select	@dashboardid,
						@projectid,
						@dashboardname,
						@authorizeduserid,
						getutcdate()
			
			select	@dashboardid dashboardid
			
			/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
			   только если это не авто список для задач из IVR (признак - отсутствие привязки к проекту)
			*/
			exec A_Policy_Objects_Save
				@idObject = @dashboardid,
				@type = 105,
				@name = @dashboardname,
				@idParentProject = @projectid,
				@actionUserId = @authorizeduserid
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

---------------------------------
-- WO_Dashboards_Dashboard_Get --
---------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Get]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Get]
	@authorizeduserid	uniqueidentifier,
	@dashboardid    uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?	
	*/
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#pp' ))
		Drop table #pp
	Create table #pp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200), [type] int)
	Insert into #pp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @projectid,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
	
	/* Привилегия чтения по дашборду есть?
	*/
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#dp' ))
		Drop table #dp
	Create table #dp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200), [type] int)
	Insert into #dp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @dashboardid,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'			
			
	
	-- Достаем из пересечения
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dp dp on dp.IdObject = d.Id
	Where (d.Id = @dashboardid)	
		and ((d.IdProject is NULL ) 
			or (exists (Select idObject From #pp pp Where pp.idObject = d.IdProject)))
	
end
GO

------------------------------------
-- WO_Dashboards_Dashboard_Remove --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Remove]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Remove]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			-- Сами дашборды
			delete
			from	[dbo].[WO_Dashboards_Dashboards]
			where	Id = @dashboardid
			
			-- Привязанные поля (для таблицы и запроса)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields in
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Привязанные поля (для конструктора)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields in
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Фильтры (для таблицы и запроса)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters in
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Фильтры (для конструктора)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters in
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Виджеты
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	IdSection in
								(
									select	Id
									from	[dbo].[WO_Dashboards_Sections]
									where	IdDashboard = @dashboardid
								)
			
			-- Борды
			delete
			from	[dbo].[WO_Dashboards_Sections]
			where	IdDashboard = @dashboardid
			
			
			-- Из таблиц политики прав доступа к объектам
			exec A_Policy_Objects_Remove @dashboardid
			
		end
		
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

------------------------------------
-- WO_Dashboards_Dashboard_Rename --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Rename]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Rename]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Rename]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier,
	@dashboardnewname	nvarchar(100)
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			-- Обновляет название
			Update [dbo].[WO_Dashboards_Dashboards]
			Set	Name = @dashboardnewname
			Where Id = @dashboardid
			
			/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
			*/
			exec A_Policy_Objects_Save
				@idObject = @dashboardid,
				@type = 105,
				@name = @dashboardnewname,
				@idParentProject = NULL, --во время переименования не волнует
				@actionUserId = @authorizeduserid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

--------------------------------------
-- WO_Dashboards_Dashboards_GetList --
--------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Dashboards_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'

	-- Доступные пользователю дашборды по привилегии ЧТЕНИЕ
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#dashboards' ))
		Drop table #dashboards
	Create table #dashboards ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))
	Insert into #dashboards ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105,
			--@idparentproject = NULL,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'

	
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dashboards p on p.IdObject = d.Id
	Where ( d.IdProject is NULL )
		or exists ( Select IdObject From #projects pr Where pr.IdObject = d.IdProject )
	Order by d.Name asc
	
end
GO

------------------------------------
-- WO_Dashboards_Projects_GetList --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Projects_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Projects_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Projects_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
			
	
	-- Достаем инфу по проектам
	Select	p.Id,
			p.Name
	From [oktell_settings].[dbo].[A_TaskManager_Projects] p
		Inner join #projects pp on pp.idObject = p.Id
	Order by p.Name asc
	
end
GO

----------------------------------
-- WO_Dashboards_Section_Create --
----------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Create]'))
DROP procedure [dbo].[WO_Dashboards_Section_Create]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Create]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier,
	@sectionname	nvarchar(100)
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			declare	@sectionid uniqueidentifier
				set	@sectionid = newid()
			
			insert into [dbo].[WO_Dashboards_Sections]
				select	@sectionid,
						@sectionname,
						getutcdate(),
						null,
						@dashboardid
			
			select	@sectionid sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

----------------------------------
-- WO_Dashboards_Section_Remove --
----------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Remove]'))
DROP procedure [dbo].[WO_Dashboards_Section_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Remove]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			delete
			from	[dbo].[WO_Dashboards_Sections]
			where	Id = @sectionid
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields in
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields in
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters in
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters in
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	IdSection = @sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

----------------------------------
-- WO_Dashboards_Section_Rename --
----------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Rename]'))
DROP procedure [dbo].[WO_Dashboards_Section_Rename]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Rename]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier,
	@sectionnewname	nvarchar(100)
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			update	[dbo].[WO_Dashboards_Sections]
			set	Name = @sectionnewname
			where	Id = @sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

------------------------------------
-- WO_Dashboards_Sections_GetList --
------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Sections_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Sections_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Sections_GetList]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
			
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
		
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	select	Id,
			Name
	from	[dbo].[WO_Dashboards_Sections]
	where	IdDashboard = @dashboardid
	order by isnull(Position, 2147483647) asc, CreationDate asc
	
end
GO

-------------------------------------------
-- WO_Dashboards_Widget_Query_Fields_Get --
-------------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Query_Fields_Get]'))
DROP procedure [dbo].[WO_Dashboards_Widget_Query_Fields_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Query_Fields_Get]
	@authorizeduserid	uniqueidentifier,
	@querytext	ntext
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			exec	('set fmtonly on ' + @querytext)
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

---------------------------------
-- WO_Dashboards_Widget_Remove --
---------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Remove]'))
DROP procedure [dbo].[WO_Dashboards_Widget_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Remove]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
		
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields =
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields =
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters =
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters =
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	Id = @widgetid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

-------------------------------
-- WO_Dashboards_Widget_Save --
-------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Save]'))
DROP procedure [dbo].[WO_Dashboards_Widget_Save]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Save]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier,
	@widgetparams	ntext
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			declare	@widgetnewid uniqueidentifier
				set	@widgetnewid = newid()
			declare	@widgetfieldsnewid uniqueidentifier
				set	@widgetfieldsnewid = newid()
			declare	@widgetfiltersnewid uniqueidentifier
				set	@widgetfiltersnewid = newid()
			declare	@widgetconstructorfieldsnewid uniqueidentifier
				set	@widgetconstructorfieldsnewid = newid()
			declare	@widgetconstructorfiltersnewid uniqueidentifier
				set	@widgetconstructorfiltersnewid = newid()
			
			declare	@widgetid uniqueidentifier
			declare	@widgetname nvarchar(100)
			declare	@widgettype int
			declare	@widgetsource int
			
			
			declare	@ixml int
			
			
			
			exec sp_xml_preparedocument @ixml output, @widgetparams
			-------------------------------------------------------
			
			-- Создаем табличные переменные для хранения данных о проверках
			declare @tempxmldata table
				(
					[id] nvarchar(50) collate database_default,
					[name] nvarchar(100) collate database_default,
					[type] nvarchar(5) collate database_default,
					[source] nvarchar(5) collate database_default,
					[table_name] nvarchar(100) collate database_default,
					[query] ntext collate database_default,
					[order_by] nvarchar(100) collate database_default,
					[show_top_rows] nvarchar(10) collate database_default,
					[options] ntext collate database_default,
					[constructor_axis_id] nvarchar(5) collate database_default
				)
			
			declare @tempxmldatafields table
				(
					[name] nvarchar(100) collate database_default,
					[value] nvarchar(5) collate database_default
				)
			
			declare @tempxmldatafilters table
				(
					[name] nvarchar(100) collate database_default,
					[operation] nvarchar(5) collate database_default,
					[value] nvarchar(100) collate database_default
				)
			
			declare @tempxmldataconstructorfields table
				(
					[field_id] nvarchar(5) collate database_default,
					[function_id] nvarchar(5) collate database_default,
					[value] nvarchar(50) collate database_default
				)
			
			declare @tempxmldataconstructorfilter table
				(
					[filter_id] nvarchar(5) collate database_default,
					[operation] nvarchar(5) collate database_default,
					[value_id] nvarchar(5) collate database_default,
					[value_1] nvarchar(50) collate database_default,
					[value_2] nvarchar(50) collate database_default
				)
			
			-- Заполняем табличные переменные данными о проверках, которые требуется выполнить
			insert into @tempxmldata
				select	*
				from openxml (@ixml,  '/root', 2)
				with	(
							[id] nvarchar(50) 'id',
							[name] nvarchar(100) 'name',
							[type] nvarchar(5) 'type',
							[source] nvarchar(5) 'source',
							[table_name] nvarchar(100) 'table_name',
							[query] ntext 'query',
							[order_by] nvarchar(100) 'order_by',
							[show_top_rows] nvarchar(10) 'show_top_rows',
							[options] ntext 'options',
							[constructor_axis_id] nvarchar(5) 'constructor_axis_id'
						)
			
			--select * from @tempxmldata
			
			
			insert into @tempxmldatafields
				select	*
				from openxml (@ixml,  '/root/table_fields', 2)
				with	(
							[name] nvarchar(100) 'name',
							[value] nvarchar(5) 'value'
						)
			
			--select * from @tempxmldatafields
			
			
			insert into @tempxmldatafilters
				select	*
				from openxml (@ixml,  '/root/table_filter', 2)
				with	(
							[name] nvarchar(100) 'name',
							[operation] nvarchar(5) 'operation',
							[value] nvarchar(100) 'value'
						)
			
			--select * from @tempxmldatafilters
			
			
			insert into @tempxmldataconstructorfields
				select	*
				from openxml (@ixml,  '/root/constructor_fields', 2)
				with	(
							[field_id] nvarchar(5) 'field_id',
							[function_id] nvarchar(5) 'function_id',
							[value] nvarchar(50) 'value'
						)
			
			--select * from @tempxmldataconstructorfields
			
			
			insert into @tempxmldataconstructorfilter
				select	*
				from openxml (@ixml,  '/root/constructor_filter', 2)
				with	(
							[filter_id] nvarchar(5) 'filter_id',
							[operation] nvarchar(5) 'operation',
							[value_id] nvarchar(5) 'value_id',
							[value_1] nvarchar(50) 'value_1',
							[value_2] nvarchar(50) 'value_2'
						)
			
			--select * from @tempxmldataconstructorfilter
			
			--------------------------------
			exec sp_xml_removedocument @ixml
			
			
				
			if ((select [id] from @tempxmldata) is null or (select [id] from @tempxmldata) = '')
				begin
					--select	'new'
					-- Widget новый (добавляем)
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets]
					insert into [dbo].[WO_Dashboards_Widgets] ([Id], [IdSection], [Name], [Type], [Source], [IdFields], [IdFilters], [NameTable], [Query], [OrderBy], [ShowTopRows], [Options], [IdConstructorFields], [IdConstructorFilters], [IdConstructorAxis])
						select	@widgetnewid [Id],
								@sectionid [IdSection],
								txmld.name [Name],
								case when txmld.type = '' then null else cast(txmld.type as int) end [Type],
								case when txmld.source = '' then null else cast(txmld.source as int) end [Source],
								@widgetfieldsnewid [IdFields],
								@widgetfiltersnewid [IdFilters],
								txmld.table_name [NameTable],
								txmld.query [Query],
								txmld.order_by [OrderBy],
								case when txmld.show_top_rows = '' then null else cast(txmld.show_top_rows as int) end [ShowTopRows],
								txmld.options [Options],
								@widgetconstructorfieldsnewid [IdConstructorFields],
								@widgetconstructorfiltersnewid [IdConstructorFilters],
								case when txmld.constructor_axis_id = '' then null else cast(txmld.constructor_axis_id as int) end [IdConstructorAxis]
						from	@tempxmldata txmld
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Fields]
					insert into [dbo].[WO_Dashboards_Widgets_Fields] ([Id], [IdFields], [Name], [Value])
						select	newid() [Id],
								@widgetfieldsnewid [IdFields],
								txmldf.name [Name],
								case when txmldf.value = '' then null else cast(txmldf.value as int) end [Value]
						from	@tempxmldatafields txmldf
						where	txmldf.name is not null and txmldf.value is not null					-- update 2013-10-30
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Filters]
					insert into [dbo].[WO_Dashboards_Widgets_Filters] ([Id], [IdFilters], [Name], [Operation], [Value])
						select	newid() [Id],
								@widgetfiltersnewid [IdFields],
								txmldfl.name [Name],
								case when txmldfl.operation = '' then null else cast(txmldfl.operation as int) end [Operation],
								txmldfl.value [Value]
						from	@tempxmldatafilters txmldfl
						where	txmldfl.name is not null and txmldfl.operation is not null					-- update 2013-10-30
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Constructor_Fields]
					insert into [dbo].[WO_Dashboards_Widgets_Constructor_Fields] ([Id], [IdConstructorFields], [ConstructorValueId], [ConstructorFunctionId], [ConstantValue])
						select	newid() [Id],
								@widgetconstructorfieldsnewid [IdConstructorFields],
								case when txmldcf.field_id = '' then null else cast(txmldcf.field_id as int) end [ConstructorValueId],
								case when txmldcf.function_id = '' then null else cast(txmldcf.function_id as int) end [ConstructorFunctionId],
								txmldcf.value [ConstantValue]
						from	@tempxmldataconstructorfields txmldcf
						where	txmldcf.function_id is not null or txmldcf.value is not null
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Constructor_Filters]
					insert into [dbo].[WO_Dashboards_Widgets_Constructor_Filters] ([Id], [IdConstructorFilters], [ConstructorFilterId], [ConstructorFilterOperationId], [ConstructorFilterValueId], [ConstructorFilterValue1], [ConstructorFilterValue2])
						select	newid() [Id],
								@widgetconstructorfiltersnewid [IdConstructorFilters],
								case when txmldcfl.filter_id = '' then null else cast(txmldcfl.filter_id as int) end [ConstructorFilterId],
								case when txmldcfl.operation = '' then 0 else cast(txmldcfl.operation as int) end [ConstructorFilterOperationId],
								case when txmldcfl.value_id = '' then null else cast(txmldcfl.value_id as uniqueidentifier) end [ConstructorFilterValueId],
								txmldcfl.value_1 [ConstructorFilterValue1],
								txmldcfl.value_2 [ConstructorFilterValue2]
						from	@tempxmldataconstructorfilter txmldcfl
						where	txmldcfl.value_id is not null or txmldcfl.value_1 is not null or txmldcfl.value_2 is not null
					
					
					select	@widgetnewid WidgetId
				end
			else
				begin
					--select	'old'
					-- Widget есть уже такой (изменяем)
					
					-- Если какой-то из параметров не высылается (null) считаем, что он не менялся
					
					-- IdSection
					if ((select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = (select cast([id] as uniqueidentifier) from @tempxmldata)) != @sectionid)
						update [dbo].[WO_Dashboards_Widgets] set [IdSection] = @sectionid
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Name
					if ((select [name] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Name] = (select [name] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Type
					if ((select [type] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Type] = (select case when [type] = '' then null else cast([type] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Source
					if ((select [source] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Source] = (select case when [source] = '' then null else cast([source] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					
					
					
					-- Проверка, есть ли изменения в fields. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldatafields) != 0)
						begin
							--select	'Есть fields'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Fields]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Fields]
												where	IdFields =
																	(
																		select	IdFields
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Fields] ([Id], [IdFields], [Name], [Value])
								select	newid() [Id],
										(
											select	IdFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdFields],
										txmldf.name [Name],
										case when txmldf.value = '' then null else cast(txmldf.value as int) end [Value]
								from	@tempxmldatafields txmldf
								-- исключая непривязанные id-ки
								where	(
											select	IdFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldf.name is not null and txmldf.value is not null					-- update 2013-10-22
						end
						
						
						
					-- Проверка, есть ли изменения в filters. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldatafilters) != 0)
						begin
							--select	'Есть filters'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Filters]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Filters]
												where	IdFilters =
																	(
																		select	IdFilters
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Filters] ([Id], [IdFilters], [Name], [Operation], [Value])
								select	newid() [Id],
										(
											select	IdFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdFilters],
										txmldfl.name [Name],
										case when txmldfl.operation = '' then null else cast(txmldfl.operation as int) end [Operation],
										txmldfl.value [Value]
								from	@tempxmldatafilters txmldfl
								-- исключая непривязанные id-ки
								where	(
											select	IdFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldfl.name is not null and txmldfl.operation is not null					-- update 2013-10-22
						end
					
					
					
					-- NameTable
					if ((select [table_name] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [NameTable] = (select [table_name] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Query
					if ((select cast([query] as nvarchar(4000)) from @tempxmldata) is not null)
						update w
						set w.[Query] = txd.[query]
						from	[dbo].[WO_Dashboards_Widgets] w, @tempxmldata txd
						where	w.[Id] = cast(txd.[id] as uniqueidentifier)
					-- OrderBy
					if ((select [order_by] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [OrderBy] = (select [order_by] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- [ShowTopRows]
					if ((select [show_top_rows] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [ShowTopRows] = (select case when [show_top_rows] = '' then null else cast([show_top_rows] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Options
					if ((select cast([options] as nvarchar(4000)) from @tempxmldata) is not null)
						update w
						set	w.[Options] = txd.[options]
						from	[dbo].[WO_Dashboards_Widgets] w, @tempxmldata txd
						where	w.[Id] = cast(txd.[id] as uniqueidentifier)
					
					
					
					-- Проверка, есть ли изменения в constructor_fields. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldataconstructorfields) != 0)
						begin
							--select	'Есть constructor_fields'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
												where	IdConstructorFields =
																	(
																		select	IdConstructorFields
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Constructor_Fields] ([Id], [IdConstructorFields], [ConstructorValueId], [ConstructorFunctionId], [ConstantValue])
								select	newid() [Id],
										(
											select	IdConstructorFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdConstructorFields],
										case when txmldcf.field_id = '' then null else cast(txmldcf.field_id as int) end [ConstructorValueId],
										case when txmldcf.function_id = '' then null else cast(txmldcf.function_id as int) end [ConstructorFunctionId],
										txmldcf.value [ConstantValue]
								from	@tempxmldataconstructorfields txmldcf
								-- исключая непривязанные id-ки
								where	(
											select	IdConstructorFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldcf.function_id is not null or txmldcf.value is not null
						end
						
						
						
					-- Проверка, есть ли изменения в constructor_filter. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldataconstructorfilter) != 0)
						begin
							--select	'Есть filters'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
												where	IdConstructorFilters =
																	(
																		select	IdConstructorFilters
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Constructor_Filters] ([Id], [IdConstructorFilters], [ConstructorFilterId], [ConstructorFilterOperationId], [ConstructorFilterValueId], [ConstructorFilterValue1], [ConstructorFilterValue2])
								select	newid() [Id],
										(
											select	IdConstructorFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdConstructorFilters],
										case when txmldcfl.filter_id = '' then null else cast(txmldcfl.filter_id as int) end [ConstructorFilterId],
										case when txmldcfl.operation = '' then 0 else cast(txmldcfl.operation as int) end [ConstructorFilterOperationId],
										case when txmldcfl.value_id = '' then null else cast(txmldcfl.value_id as uniqueidentifier) end [ConstructorFilterValueId],
										txmldcfl.value_1 [ConstructorFilterValue1],
										txmldcfl.value_2 [ConstructorFilterValue2]
								from	@tempxmldataconstructorfilter txmldcfl
								-- исключая непривязанные id-ки
								where	(
											select	IdConstructorFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldcfl.value_id is not null or txmldcfl.value_1 is not null or txmldcfl.value_2 is not null
						end
					
					
					
					-- ConstructorAxisId
					if ((select [constructor_axis_id] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [IdConstructorAxis] = (select case when [constructor_axis_id] = '' then null else cast([constructor_axis_id] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					
					
				end
				
				
		end
		
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

-----------------------------------------------
-- WO_Dashboards_Widget_Table_Fields_GetList --
-----------------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Table_Fields_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Widget_Table_Fields_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Table_Fields_GetList]
	@authorizeduserid	uniqueidentifier,
	@tablename	nvarchar(1000)
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			select	f.name FieldName
			from	[dbo].[sysobjects] t, [dbo].[syscolumns] f
			where	t.id = f.id
					and t.type in ('U', 'V')
					and t.name = @tablename
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

-----------------------------------------
-- WO_Dashboards_Widget_Tables_GetList --
-----------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Tables_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Widget_Tables_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Tables_GetList]
	@authorizeduserid	uniqueidentifier
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			select	t.name TableName
			from	[dbo].[sysobjects] t
			where	t.type in ('U', 'V')
			order by t.name asc
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO

-------------------------------------
-- WO_Dashboards_Widgets_GetConfig --
-------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetConfig]'))
DROP procedure [dbo].[WO_Dashboards_Widgets_GetConfig]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetConfig]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
	
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
	
	if (@dcount= 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	select	dw.NameTable TableName,
			dw.Query
	from	[dbo].[WO_Dashboards_Widgets] dw
	where	dw.Id = @widgetid
	
	select	wf.Name WidgetFilterName,
			wf.Operation WidgetFilterOperation,
			wf.Value WidgetFilterValue
	from	[dbo].[WO_Dashboards_Widgets] w 
				inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters
	where	w.Id = @widgetid
											-- привязать к словарю?
	select	wcf.ConstructorFilterId,
			wcf.ConstructorFilterOperationId,
			wcf.ConstructorFilterValueId,
			wcf.ConstructorFilterValue1,
			wcf.ConstructorFilterValue2
	from	[dbo].[WO_Dashboards_Widgets] w 
				inner join [dbo].[WO_Dashboards_Widgets_Constructor_Filters] wcf on w.IdConstructorFilters = wcf.IdConstructorFilters
	where	w.Id = @widgetid
	
end
GO

-----------------------------------
-- WO_Dashboards_Widgets_GetData --
-----------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetData]'))
DROP procedure [dbo].[WO_Dashboards_Widgets_GetData]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetData]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
	
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
	
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	-- конструктор
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 0)
		begin
			
			--select	'конструктор'
			
			declare	@sqldesigner nvarchar(4000)
			
			
			
			declare	@constructorvalues nvarchar(4000)
			set	@constructorvalues = ''
			select	@constructorvalues = @constructorvalues +
					case
						when cv.ConstructorValueId = 1 then
							case
								when cv.ConstructorFunctionId = 1 then 'round(avg(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 2 then 'round(sum(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 3 then 'round(max(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 4 then 'round(min(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
							end
					end
			from
			(
				select	cf.ConstructorValueId,
						dv.Value,
						cf.ConstructorFunctionId,
						df.[Function],
						df.FunctionText
				from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf
							inner join [dbo].[WO_Dashboards_Widgets_Designer_Function] df on cf.ConstructorFunctionId = df.Id
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id
				where	cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
			) cv
			
			--select @constructorvalues
			
			declare	@constructorvalues1 nvarchar(4000)
			set	@constructorvalues1 = ''
			select	@constructorvalues1 = @constructorvalues1 +
					case
						when cv.ConstructorValueId = 1 then
							case
								when cv.ConstructorFunctionId = 1 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 2 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 3 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
								when cv.ConstructorFunctionId = 4 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
							end
					end
			from
			(
				select	cf.ConstructorValueId,
						dv.Value,
						cf.ConstructorFunctionId,
						df.[Function],
						df.FunctionText
				from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf
							inner join [dbo].[WO_Dashboards_Widgets_Designer_Function] df on cf.ConstructorFunctionId = df.Id
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id
				where	cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
			) cv
			
			--select @constructorvalues1
			
			declare	@constructorfilters nvarchar(4000)
			set	@constructorfilters = 'and '
			select	@constructorfilters = @constructorfilters +
					case
						when cf.ConstructorFilterId = 1 then
							case
								when (select Id from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 1 then 'cast(floor(cast(effconn_1.DateTimeStart as float)) as datetime)'
								else 'effconn_1.DateTimeStart'
							end
						when cf.ConstructorFilterId = 2 then '_время_'
						when cf.ConstructorFilterId = 3 then '_оператор_'
						when cf.ConstructorFilterId = 4 then '_задача_'
						when cf.ConstructorFilterId = 5 then '_проект_'
					end
					
					+ ' ' +
					
					case
						when cf.ConstructorFilterOperationId = 0 then
							case
								when cf.ConstructorFilterValueId is not null then '='
								else 'between'
							end
					end
					
					+ ' ' +
					
					case
						when cf.ConstructorFilterId = 1 then
							case
								when cf.ConstructorFilterValueId is not null then
									case
										when (select Id from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 1 then 'cast(floor(cast(getdate() as float)) as datetime)'
									end
								else 'cast(''' + cf.ConstructorFilterValue1 + ' ' + '00:00:00.000' + ''' as datetime) and ''' + cf.ConstructorFilterValue2 + ' ' + '23:59:59.900' + ''''
							end
						when cf.ConstructorFilterId  in (4) then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
					end
					
					+ ' and '
			from
			(
				select	ConstructorFilterId,
						ConstructorFilterOperationId,
						ConstructorFilterValueId,
						ConstructorFilterValue1,
						ConstructorFilterValue2
				from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
				where	IdConstructorFilters = (select IdConstructorFilters from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
			) cf
			
			--select @constructorfilters
			
			-- шаблон для подсчета значения
			if ((select distinct ConstructorValueId from [dbo].[WO_Dashboards_Widgets_Constructor_Fields] where IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)) = 1)
				begin
					--select	'Cредняя скорость ответа (ASA)'
					select	distinct
							@sqldesigner = dwdv.ModelQuery
					from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] dwcf
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dwdv on dwcf.ConstructorValueId = dwdv.Id
					where	dwcf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
				end
			
			-- в зависимости от оси добавляем столбец
			select @sqldesigner =
				case w.IdConstructorAxis
					when 1 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' — '' + convert(nvarchar(5), dateadd(mi, 5, cast((floor(cast(effconn_2.TimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
					when 2 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' — '' + convert(nvarchar(5), dateadd(mi, 15, cast((floor(cast(effconn_2.TimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
					when 3 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' — '' + convert(nvarchar(5), dateadd(mi, 30, cast((floor(cast(effconn_2.TimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
					when 4 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' — '' + convert(nvarchar(5), dateadd(hh, 1, cast((floor(cast(effconn_2.TimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
					when 5 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime), 4) + ''.'' + cast(datepart(yy, cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime)) as nvarchar(4)) ' + da.Axis)
					when 6 then replace(@sqldesigner, '_select_', 'convert(nvarchar(5), cast(floor(cast(effconn_2.DateTimeStart as float) - datepart(dw, effconn_2.DateTimeStart) + 1) as datetime), 4) + '' — '' + convert(nvarchar(5), dateadd(dd, 6, cast(floor(cast(effconn_2.DateTimeStart as float) - datepart(dw, effconn_2.DateTimeStart) + 1) as datetime)), 4) ' + da.Axis)
      				when 7 then replace(@sqldesigner, '_select_', '
      							case datepart(mm, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime))
      								when 1 then ''Январь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 2 then ''Февраль '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 3 then ''Март '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 4 then ''Апрель '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 5 then ''Май '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 6 then ''Июнь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 7 then ''Июль '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 8 then ''Август '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 9 then ''Сентябрь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 10 then ''Октябрь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 11 then ''Ноябрь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      								when 12 then ''Декабрь '' + cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4))
      							end ' + da.Axis)
      				when 8 then replace(@sqldesigner, '_select_', 'cast(datepart(yy, cast(cast(datepart(yy, getdate()) as nvarchar (4)) + ''-01-01'' as datetime)) as nvarchar(4)) + '' год'' ' + da.Axis)
					when 9 then replace(@sqldesigner, '_select_', 'effconn_1.IdOperator ')
					when 10 then replace(@sqldesigner, '_select_', 'effconn_1.IdTask ')
					when 11 then replace(@sqldesigner, '_select_', 'effconn_1.IdProject ')
					when 12 then replace(@sqldesigner, '_select_', '
      							case datepart(dw, effconn_2.DateTimeStart)
      								when 1 then ''Понедельник''
      								when 2 then ''Вторник''
      								when 3 then ''Среда''
      								when 4 then ''Четверг''
      								when 5 then ''Пятница''
      								when 6 then ''Суббота''
      								when 7 then ''Воскресенье''
      							end ' + da.Axis)
				end
			from	[dbo].[WO_Dashboards_Widgets] w
						inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
			where	w.Id = @widgetid
			
			-- в зависимости от оси добавляем прнивязку
			set @sqldesigner =
				case (select IdConstructorAxis from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
					when 9 then replace(@sqldesigner, '_join_', 'inner join (select IdChain, LenQueue, min(DateTimeStart) minDateTimeStart from [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] where IdOperator not in (''AB000000-0000-0000-0000-000000000000'', ''BF000000-0000-0000-0000-000000000000'', ''00000000-0000-0000-0000-000000000000'') and IdOperator is not null group by IdChain, LenQueue) effconn_3
																	on effconn_1.IdChain = effconn_3.IdChain and effconn_1.LenQueue = effconn_3.LenQueue and effconn_1.DateTimeStart = effconn_3.minDateTimeStart')
					else replace(@sqldesigner, '_join_', '')
				end +
				case
					when @constructorfilters = 'and ' then ''
					else ' ' + (left(@constructorfilters, len(@constructorfilters)-len(' and ')))
				end
			
			-- в зависимости от значений вычисляем их
			select @sqldesigner =
				'SET DATEFIRST 1 ' +
				case w.IdConstructorAxis
					when 9 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' from ('
					when 10 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' from ('
					when 11 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' from ('
					else ''
				end
				+ 'select ' +
				case w.IdConstructorAxis
					when 1 then 'lq.' + da.Axis
					when 2 then 'lq.' + da.Axis
					when 3 then 'lq.' + da.Axis
					when 4 then 'lq.' + da.Axis
					when 5 then 'lq.' + da.Axis
					when 6 then 'lq.' + da.Axis
					when 7 then 'lq.' + da.Axis
					when 8 then 'lq.' + da.Axis
					when 9 then 'lq.IdOperator, oi.Name ' + da.Axis
					when 10 then 'lq.IdTask, t.Name ' + da.Axis
					when 11 then 'lq.IdProject, p.Name ' + da.Axis
					when 12 then 'lq.' + da.Axis
				end
				+ ', ' +
				(left(@constructorvalues, len(@constructorvalues)-len(', ')))
				+
				' from ('
				+ @sqldesigner +
				') lq ' +
				case w.IdConstructorAxis
					when 9 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi on lq.IdOperator = oi.Id '
					when 10 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] t on lq.IdTask = t.Id '
					when 11 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] p on lq.IdProject = p.Id '
					else ''
				end
				+ 'group by ' +
				case w.IdConstructorAxis
					when 1 then 'lq.' + da.Axis
					when 2 then 'lq.' + da.Axis
					when 3 then 'lq.' + da.Axis
					when 4 then 'lq.' + da.Axis
					when 5 then 'lq.' + da.Axis
					when 6 then 'lq.' + da.Axis
					when 7 then 'lq.' + da.Axis
					when 8 then 'lq.' + da.Axis
					when 9 then 'lq.IdOperator, oi.Name'
					when 10 then 'lq.IdTask, t.Name'
					when 11 then 'lq.IdProject, p.Name'
					when 12 then 'lq.' + da.Axis
				end
				+
				case w.IdConstructorAxis
					when 9 then ') lq'
					when 10 then ') lq'
					when 11 then ') lq'
					else ''
				end
			from	[dbo].[WO_Dashboards_Widgets] w
						inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
			where	w.Id = @widgetid
			
			
			
			--select @sqldesigner
			
			exec(@sqldesigner)
		end
	
	
	-- таблица бд
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 1)
		begin
			--select	'таблица бд'
			
			declare	@sqltable nvarchar(4000)
			
			
			declare	@fields nvarchar(4000)
			set	@fields = ''
			select	@fields = @fields + wf.Name + ', '
			from	[dbo].[WO_Dashboards_Widgets] w 
						inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
			where	w.Id = @widgetid
			
			declare	@filters nvarchar(4000)
			set	@filters = ''
			select	@filters = @filters + w.OperationTextValue + ' and ' from
			(
				select	wf.Name +
						' ' +
						case
							when wfo.Id in (4,5,6,7) then replace(wfo.OperationText, '_', wf.Value)
							else wfo.OperationText + ' ''' + wf.Value + ''''
						end OperationTextValue
				from	[dbo].[WO_Dashboards_Widgets] w
							inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters
							inner join [dbo].[WO_Dashboards_Widgets_Filters_Operation] wfo on wf.Operation = wfo.Id
				where	w.Id = @widgetid
			) w
			
			
			
			set	@sqltable =
				case
					when ( select count(*) 
						   from [dbo].[WO_Dashboards_Widgets] w 
								inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields 
						   where w.Id = @widgetid and wf.Name is not null and wf.Value is not null) = 0 
						then 'select null'
					else
						'select ' +
							case 
								when (( select count(*) 
										from [dbo].[WO_Dashboards_Widgets] w 
										where (w.Id = @widgetid) 
											and (w.Type = 4) 
											and (w.ShowTopRows is not null)) != 0 ) 
									then 'top(' + cast((select w.ShowTopRows from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) as varchar) + ') '
								else ''
							end +
						(left(@fields, len(@fields)-len(', '))) +
						' from ' +
						(
							select	w.NameTable
							from	[dbo].[WO_Dashboards_Widgets] w
							where	w.Id = @widgetid
						) +
						case
							when (( select count(*) 
								   from [dbo].[WO_Dashboards_Widgets] w 
										inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters 
								   where (w.Id = @widgetid) and (wf.Name is not null) and (wf.Operation is not null)) = 0 ) 
								then ''
							else
								' where ' + (left(@filters, len(@filters)-len(' and ')))
						end +
						case
							when (( select count(*) 
									from [dbo].[WO_Dashboards_Widgets] w 
									where (w.Id = @widgetid) and (w.Type = 4) and (w.OrderBy is not null) and (w.OrderBy != '')) != 0 ) 
								then ' order by ' + (select w.OrderBy from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid)
							else ''
						end
				end
			
						
			exec(@sqltable)
		end
	
	
	-- sql-запрос
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 2)
		begin
			
			--select	'sql-запрос'
			
			declare	@sqlquery nvarchar(4000)
			set	@sqlquery =
			(
				select	cast(w.Query as nvarchar(4000))
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			
			exec(@sqlquery)
		end
	
end
GO

-----------------------------------
-- WO_Dashboards_Widgets_GetList --
-----------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetList]'))
DROP procedure [dbo].[WO_Dashboards_Widgets_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetList]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier
as
begin
	
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = @sectionid
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
			
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
		
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	
	select	w.Id,
			w.Name,
			w.[Type],
			w.[Source],
			w.Options
	from	[dbo].[WO_Dashboards_Widgets] w
	where	w.IdSection = @sectionid
	
	
	declare @widgets table
		(
			WidgetId uniqueidentifier,
			WidgetFieldId uniqueidentifier,
			WidgetFieldCode nvarchar(100) collate database_default,
			WidgetFieldName nvarchar(250) collate database_default,
			WidgetFieldValue int
		)
	
	if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (1,2)) != 0)
		begin
			insert into @widgets (WidgetId, WidgetFieldId, WidgetFieldCode, WidgetFieldName, /*WidgetFunctionCode, WidgetFunctionName,*/ WidgetFieldValue)
				select	w.Id WidgetId,
						null WidgetFieldId,
						null WidgetFieldCode,
						wf.Name WidgetFieldName,
						wf.Value WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
				where	w.IdSection = @sectionid
						and w.[Source] in (1,2)
		end
	
	if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (0)) != 0)
		begin
			insert into @widgets (WidgetId, WidgetFieldId, WidgetFieldCode, WidgetFieldName, /*WidgetFunctionCode, WidgetFunctionName,*/ WidgetFieldValue)
				select	w.Id WidgetId,
						wdv.IdUniqueidentifier WidgetFieldId,
						wdv.Value WidgetFieldCode,
						lt1.langvalue WidgetFieldName,
						1 WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] wcf on w.IdConstructorFields = wcf.IdConstructorFields
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] wdv on wdv.Id = wcf.ConstructorValueId
								inner join [oktell_settings].[dbo].[A_LangTexts] lt1 on wdv.Value = lt1.langkey
				where	w.IdSection = @sectionid
						and w.[Source] in (0)
				
					union all
				
				select	w.Id WidgetId,
						wda.IdUniqueidentifier WidgetFieldId,
						wda.Axis WidgetFieldCode,
						lt.langvalue WidgetFieldName,
						0 WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] wda on w.IdConstructorAxis = wda.Id
								inner join [oktell_settings].[dbo].[A_LangTexts] lt on wda.Axis = lt.langkey
				where	w.IdSection = @sectionid
						and w.[Source] in (0)
		end
	
	
	select	WidgetId,
			WidgetFieldId,
			WidgetFieldCode,
			WidgetFieldName,
			WidgetFieldValue
	from	@widgets
	
end
GO