﻿if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Stat_Service_FillTempByFilter' ))
	Drop procedure dbo.A_Stat_Service_FillTempByFilter
GO


Create Procedure [dbo].[A_Stat_Service_FillTempByFilter]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = NULL, --Task defined
	@fromcc bit = NULL,
	@operatorid uniqueidentifier = NULL, --Operator defined
	@innerprefix nvarchar(100) = NULL, --Internal Number defined
	@outnumber nvarchar(100) = NULL,  --Out Number defined
	@conntype int = -1,
	@showconference bit = 0,
	@showfailed bit = 0,
	@comment nvarchar ( 4000 ) = NULL,
	@idconn uniqueidentifier = '00000000-0000-0000-0000-000000000000',
	@idchain uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin

----------------------------------------------
------- Подготовка параметров ----------------
----------------------------------------------

	Declare @filtercomment bit
	Set @filtercomment = case when ( @comment IS NULL or @comment like '' ) then 0 else 1 end

	Declare @tstartfloat float  
	Declare @tstopfloat float 
	Set @tstartfloat = Convert ( float, @tstart )  
	Set @tstopfloat  = Convert ( float, @tstop )  
	Set @tstartfloat = @tstartfloat - FLOOR ( @tstartfloat )  
	Set @tstopfloat = @tstopfloat - FLOOR ( @tstopfloat )  
	If ( @tstopfloat <= @tstartfloat )
	    Set @tstopfloat = @tstopfloat + 1

	Declare @lt float
	Set @lt = Convert ( float, @lentime ) - Floor ( Convert ( float, @lentime ) )

	Declare @idempty uniqueidentifier
	Set @idempty = '00000000-0000-0000-0000-000000000000'
	
	Declare @idconfuser uniqueidentifier
	Set @idconfuser = 'CF000000-0000-0000-0000-000000000000'

--------------
--------------

	Declare @intnumbers table ( nid uniqueidentifier, prefix nvarchar ( 50 ) collate database_default, addrtype int, reactid uniqueidentifier )

	if ( @innerprefix is not null ) begin
		
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#t_c' ))
			Drop table #t_c
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#temp_c' ))
			Drop table #temp_c
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#removed_c' ))
			Drop table #removed_c
		Create table #t_c ( nid uniqueidentifier, reactid uniqueidentifier )
		Create table #temp_c ( id uniqueidentifier, oldrid uniqueidentifier, newrid uniqueidentifier )
		Create table #removed_c ( nid uniqueidentifier, reactid uniqueidentifier )

		Insert Into #t_c
			Select np.Id, rr.ReactId
			From A_NumberPlan np
				Inner Join A_NumberPlanAction na on na.NumId = np.Id
				Inner Join A_Rules r on na.ExtraId = r.Id
				Inner Join A_RuleRecords rr on rr.RuleId = r.Id
			Where na.NumType = 0 --только стандартные (расчет отделов по входящим в состав элементам)

		
		Declare @cnt int, @rowcount int, @msg int
		Select @cnt = 0, @rowcount = 1
		
		While ( @rowcount > 0 ) and ( @cnt < 10 )
		begin
			Set @cnt = @cnt + 1
		
			--Выгребаем ссылки на номера @cnt - уровня
			Insert Into #temp_c
				Select t.nid, t.reactid, t1.reactid
				From #t_c t
					 Inner Join #t_c t1 on ( t.Reactid = t1.nId ) and ( t1.nId != t.nId ) and ( t1.reactid != t.nId )
				Where t.ReactId != t.Nid
					and not exists ( select nid from #removed_c r where t.nid = r.nid and t1.reactid = r.reactid )
					and not exists ( select nid from #t_c r where t.nid = r.nid and t1.reactid = r.reactid )
				Group by t.nid, t.reactid, t1.reactid
			
			-- 	
			Insert into #removed_c ( nid, reactid )
				Select t.nid, t.reactid
				From #t_c t
					Inner join #temp_c tmp on t.nid= tmp.id and t.reactid = tmp.oldrid
				Where not exists ( select nid from #removed_c r where t.nid = r.nid and t.reactid = r.reactid )
				Group by t.nid, t.reactid
					
								
			--Удаляем то что будем менять
			Delete From #t_c
				From #temp_c
			Where nid = id and reactid = oldrid


			--вставляем замены
			Insert Into #t_c ( nid, reactid )
				Select id, newrid From #temp_c

			--чистим времянку
			Select @rowcount = COUNT ( * ) 
			From #temp_c
					
			Truncate table #temp_c
		end
			
		-- удаляем остатки ссылок на номера
		Delete From #t_c
			From A_NumberPlan np
		Where reactid = np.Id

		-- Вставляем результат
		Declare @rr table ( ReactId uniqueidentifier, InnerAddressType int )

		Insert into @rr ( ReactId, InnerAddressType )
			Select ReactId, Min ( InnerAddressType )
			From A_RuleRecords rr 
			Group by ReactId

		Insert into @intnumbers ( nid, prefix, addrtype, reactid )
			Select t.nid, np.Prefix, rr.InnerAddressType, t.reactid
			From #t_c t
				Inner join A_NumberPlan np on t.nid = np.Id				
				Inner join @rr rr on rr.ReactId = t.ReactId

	end		

--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ----------------------------------------------------
    --- Коммутации -------------------------------------
    ----------------------------------------------------

	-- Временные таблицы для коммутаций
	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Connections_1x1_tmp2' ))
		Drop table #A_Stat_Connections_1x1_tmp2
	Create table #A_Stat_Connections_1x1_tmp2 ( Id uniqueidentifier Primary Key )

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Connections_1x1_tmp' ))
		Drop table #A_Stat_Connections_1x1_tmp
	Create table #A_Stat_Connections_1x1_tmp ( Id uniqueidentifier PRIMARY KEY, AUSerId uniqueidentifier, BUSerId uniqueidentifier, ALineId uniqueidentifier, BLineId uniqueidentifier )

    -----------------------------------------------
    ------- Начальная выборка коммутаций ----------
    -----------------------------------------------

	if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

		-- Date & Time Intervals
		-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
		--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))

	end else if ( @idchain = '00000000-0000-0000-0000-000000000000' ) begin
		
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( Id = @idconn )
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	
	end else if ( @idconn = '00000000-0000-0000-0000-000000000000' ) begin
		
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( IdChain = @idchain )			
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	
	end else begin

		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( IdChain = @idchain )
			and ( Id = @idconn )		
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	end

    -------------------------------------------------------
    ------- Фильтр по правам контроля (префильтр) ---------
    -------------------------------------------------------

	Declare @commpermitted table ( Id uniqueidentifier PRIMARY KEY ) -- Id Connection

	------------------------------------
	-- По пользователям
	------------------------------------

	Declare @upermitted table ( Id uniqueidentifier Primary Key )
	
	Insert into @upermitted
		Select UserB
		From A_UserControls auc
		Where UserA = @iduser

	If not exists ( Select UserB From A_UserControls Where ( UserA = @iduser ) and ( UserB = @iduser )) 
		Insert into @upermitted ( Id ) 
		Values ( @iduser )

	Insert into @commpermitted
		Select sc.ID --auc.UserB   	
		From @upermitted up
			Inner join #A_Stat_Connections_1x1_tmp sc on up.Id = sc.AUserId
	
	Insert into @commpermitted
		Select sc.ID --auc.UserB   	
		From @upermitted up
			Inner join #A_Stat_Connections_1x1_tmp sc on up.Id = sc.BUserId
		Where not exists ( Select id From @commpermitted Where Id = sc.ID)

	------------------------------------
	-- По линиям
	------------------------------------

	Declare @lpermitted table ( Id uniqueidentifier Primary Key )
	
	-- Внутренние линии
	Insert into @lpermitted
	    Select IdLine
	    From A_UserControlLines ucl
	    Where ( UserA = @iduser )
	    	and not exists ( Select id From A_CallRoutes cr Where cr.Id = ucl.IdLine )
      		and ( exists ( Select id From A_ServerIntLines l Where l.Id = ucl.IdLine )
      			or exists ( Select id From A_ServerSipLines l Where l.Id = ucl.IdLine ))
      		--and ( IdLine != '11111111-1111-1111-1111-111111111111' )
      		--and ( IdLine != '6F5F7372-D82E-4FB6-97CB-85487A85067C' )
      		--and ( IdLine != '14B559C8-BDF8-4D17-9968-FEB8F9F54F47' )
              	  	
    -- По направлениям внешним
    Insert into @lpermitted
		Select Distinct crl.IdLine
		From A_UserControlLines ucl 
			Inner join A_CallRoutes_Lines crl on crl.IdRoute = ucl.IdLine
		Where (ucl.UserA = @iduser)
			And not exists ( Select Id from @lpermitted lp Where lp.Id = crl.IdLine )

	-- Все внешние
	Declare @canlistenext int       
	Select @canlistenext = Count ( * ) 
	From A_UserControlLines 
	Where ( UserA = @iduser ) 
		and (( IdLine = '11111111-1111-1111-1111-111111111111' )
			 or ( IdLine = '6F5F7372-D82E-4FB6-97CB-85487A85067C' ))

	if ( @canlistenext > 0 ) begin
		Insert into @lpermitted
            Select Id
    	    From A_ServerExtLines sel
            Where not exists ( Select id From @lpermitted lp Where lp.Id = sel.ID )

		Insert into @lpermitted ( Id ) 
		Values ( 'BF000000-0000-0000-0000-000000000000' )
	end
	
	-- Все межсерверные
	Declare @canlistenxch int       
	Select @canlistenxch = Count ( * ) 
	From A_UserControlLines 
	Where ( UserA = @iduser ) 
		and ( IdLine = '14B559C8-BDF8-4D17-9968-FEB8F9F54F47' )			 

	if ( @canlistenxch > 0 ) begin
		Insert into @lpermitted
            Select Id
    	    From A_ServerExtLines sel
            Where ([Type] = 13)
				and not exists ( Select id From @lpermitted lp Where lp.Id = sel.ID )
	end
     
    ----
	Insert into @commpermitted
	    Select distinct sc.ID            
	    From @lpermitted l
			Inner join #A_Stat_Connections_1x1_tmp sc on ( sc.ALineId = l.Id ) or ( sc.AUserId = l.Id )
	    Where not exists ( Select id From @commpermitted Where Id = sc.ID )

	Insert into @commpermitted            
	    Select distinct sc.ID            
	    From @lpermitted l
      		Inner join #A_Stat_Connections_1x1_tmp sc on ( sc.BLineId = l.Id ) or ( sc.BUserId = l.Id )
	    Where not exists ( Select id From @commpermitted Where Id = sc.ID)
	    	    
	------------------------------------
	-- По хештегам (цепочки и коммутации)
	------------------------------------
	
	------------------------------------

	Delete 
	From #A_Stat_Connections_1x1_tmp 
	Where id not in ( Select Id from @commpermitted )


    -------------------------------------------------------
    ------- Фильтр по номерам (постфильтр) ----------------
    -------------------------------------------------------

	----------------------------------
	-- Внутренние номера
	----------------------------------

	if ( @innerprefix is not null ) begin
		
		--Users
		Insert into #A_Stat_Connections_1x1_tmp2 
  		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.AUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.BUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where id = sc.ID )

		--Lines
		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.ALineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where Id = sc.ID )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.BLineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where Id = sc.ID )
		
		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where Id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )

		Truncate table #A_Stat_Connections_1x1_tmp2
		
	end 

	---------------------------------------
	-- Внешние номера
	---------------------------------------

	if ( @OutNumber is not null ) begin

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From A_Stat_Connections_1x1 sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( AOutNumber like '%'+@OutNumber+'%' )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From A_Stat_Connections_1x1 sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( BOutNumber like '%'+@OutNumber+'%' )
			and not exists ( Select id From #A_Stat_Connections_1x1_tmp2 Where id = sc.ID )

		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )

		Truncate table #A_Stat_Connections_1x1_tmp2

	end

    -----------------------------------------------------------------
    ------- Фильтр по пользователю (постфильтр) ---------------------
    -----------------------------------------------------------------

	if ( @operatorid is not null ) begin

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
		    Where AUserId = @operatorid

		Insert into #A_Stat_Connections_1x1_tmp2 
		select Id 
		from #A_Stat_Connections_1x1_tmp sc
		where BUserId = @operatorid
			and not exists ( Select id From #A_Stat_Connections_1x1_tmp2 where id = sc.ID )

		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )
		
		Truncate table #A_Stat_Connections_1x1_tmp2
	end

    ---------------------------------------------------------------
    -- Фильтр по комментарию (если установлен) (постфильтр) -------
    ---------------------------------------------------------------

	if ( @filtercomment > 0 ) begin

		Delete 
		From #A_Stat_Connections_1x1_tmp
		Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )
	end


--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ------------------------------------------------
    ------- Неудачные исходящие --------------------
    ------------------------------------------------

    if (( @showfailed = 1 ) and ( @lt = 0 )) begin

	-- Временные таблицы для конференций
	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Failed_tmp' ))
		Drop table #A_Stat_Failed_tmp
	Create table #A_Stat_Failed_tmp ( Id uniqueidentifier PRIMARY KEY, AUSerId uniqueidentifier, ALineId uniqueidentifier )

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Failed_tmp2' ))
		Drop table #A_Stat_Failed_tmp2
	Create table #A_Stat_Failed_tmp2 ( Id uniqueidentifier PRIMARY KEY )

    ------------------------------------------------
    ------- Начальная выборка неудачных  ----------
    ------------------------------------------------
	
	if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

		-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
		--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
		Insert into #A_Stat_Failed_tmp
			Select sc.Id, sc.AUserId, sc.ALineId
			From A_Stat_FailedCalls sc
			Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
				and ( Convert ( float, TimeStart ) - Convert ( float, TimeStart ) >= @lt )
	end

    -----------------------------------------------------
    ------- Фильтр по правам контроля (префильтр) -------
    -----------------------------------------------------

	------------------------------------
	-- По пользователям
	------------------------------------

	Insert into @commpermitted
	    Select sc.ID
	    From @upermitted up
			Inner join #A_Stat_Failed_tmp sc on up.Id = sc.AUserId

	------------------------------------
	-- По линиям
	------------------------------------

	Insert into @commpermitted
	    Select distinct sc.ID            
	    From @lpermitted l
			Inner join #A_Stat_Failed_tmp sc on ( sc.ALineId = l.Id ) or ( sc.AUserId = l.Id )
	    Where not exists ( Select id From @commpermitted Where Id = sc.ID )

	------------------------------------
	-- Удаляем те что нельзя
	------------------------------------

	Delete 
	From #A_Stat_Failed_tmp 
	Where id not in ( Select Id from @commpermitted )

    ---------------------------------------------------------
    -------- Фильтр по номерам (постфильтр) -----------------
    ---------------------------------------------------------

	------------------------------------
	-- Внутренние номера
	------------------------------------

	if ( @innerprefix is not null ) begin

		--Users
		Insert into #A_Stat_Failed_tmp2 
  		    Select Id
		    From #A_Stat_Failed_tmp sc
				Inner join @intnumbers t on sc.AUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )

		--Lines
		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From #A_Stat_Failed_tmp sc
				Inner join @intnumbers t on sc.ALineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Failed_tmp2 Where Id = sc.ID )

		
		Delete #A_Stat_Failed_tmp 
		Where Id not in ( Select Id From #A_Stat_Failed_tmp2 )

		Truncate table #A_Stat_Failed_tmp2
	end

	---------------------------------------
	-- Внешние номера
	---------------------------------------

	if ( @OutNumber is not null ) begin

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From A_Stat_FailedCalls sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( AOutNumber like '%'+@OutNumber+'%' )

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From A_Stat_FailedCalls sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( ANumberDialed like '%'+@OutNumber+'%' )
			and not exists ( Select id From #A_Stat_Failed_tmp2 Where id = sc.ID )

		Delete 
		From #A_Stat_Failed_tmp 
		Where id not in ( Select Id From #A_Stat_Failed_tmp2 )

		Truncate table #A_Stat_Failed_tmp2

	end

    -------------------------------------------------------------
    ------- Фильтр по пользователю (постфильтр) -----------------
    -------------------------------------------------------------

	if ( @operatorid is not null ) begin

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From #A_Stat_Failed_tmp sc
		    Where AUserId = @operatorid


		Delete 
		From #A_Stat_Failed_tmp 
		Where id not in ( Select Id From #A_Stat_Failed_tmp2 )
		
		Truncate table #A_Stat_Failed_tmp2
	end

    ----------------------------------------------------------
    -- Фильтр по комментарию (если установлен) (постфильтр) --
    ----------------------------------------------------------

	if ( @filtercomment > 0 ) begin

		Delete 
		From #A_Stat_Failed_tmp
		Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )
	  end

    end


--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ------------------------------------------------
    ------- Конференции ----------------------------
    ------------------------------------------------

    if ( @showconference = 1 ) begin

		-- Временные таблицы для конференций
		if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp' ))
			Drop table #A_Stat_Conference_tmp
		Create table #A_Stat_Conference_tmp ( Id uniqueidentifier PRIMARY KEY, RecordAccess int, Director uniqueidentifier )

		if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp2' ))
			Drop table #A_Stat_Conference_tmp2
		Create table #A_Stat_Conference_tmp2 ( Id uniqueidentifier PRIMARY KEY )

		------------------------------------------------
		------- Начальная выборка конференций ----------
		------------------------------------------------

		if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

			-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
			--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
			Insert into #A_Stat_Conference_tmp
				Select sc.Id, sc.RecordAccess, sc.Director 
				From A_Stat_Conference sc
				Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
					and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
					and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
					and ( Convert ( float, TimeStop ) - Convert ( float, TimeStart ) >= @lt )
					and (( @conntype = -1 ) or ( @conntype = 4 ))

		end else if ( @idchain = '00000000-0000-0000-0000-000000000000' ) begin

			Insert into #A_Stat_Conference_tmp
				Select sc.Id, sc.RecordAccess, sc.Director 
				From A_Stat_Conference sc
					Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
				Where ( scc.IdConn = @idconn )
					and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
					and (( @conntype = -1 ) or ( @conntype = 4 ))

		end else if ( @idconn = '00000000-0000-0000-0000-000000000000' ) begin	
		
			Insert into #A_Stat_Conference_tmp
				Select sc.Id, sc.RecordAccess, sc.Director 
				From A_Stat_Conference sc
					Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
				Where ( sc.Id = @idchain )
					and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
					and (( @conntype = -1 ) or ( @conntype = 4 ))
					
		end else begin

			Insert into #A_Stat_Conference_tmp
				Select sc.Id, sc.RecordAccess, sc.Director 
				From A_Stat_Conference sc
					Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
				Where ( sc.Id = @idchain )
					and ( scc.IdConn = @idconn )
					and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
					and (( @conntype = -1 ) or ( @conntype = 4 ))
		end


		--------------------------------------------------------
		------- Фильтры ----------------------------------------
		--------------------------------------------------------

		------------------------------------------------------------
		-- Фильтр по комментарию (если установлен) (префильтр) -----
		------------------------------------------------------------

		if ( @filtercomment > 0 )	
			Delete 
			From #A_Stat_Conference_tmp
			Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )		

		------------------------------------------------
		-- Фильтр по правам (префильтр)
		------------------------------------------------

		-----------------------------
		-- права самой конференции --
		-----------------------------

		Insert into #A_Stat_Conference_tmp2
			Select Distinct sc.Id 
			From #A_Stat_Conference_tmp sc
				Inner join @upermitted up on sc.Director = up.Id
			Where sc.RecordAccess = 1 --Директор слушает 

		Insert into #A_Stat_Conference_tmp2
			Select sc.Id 
			From #A_Stat_Conference_tmp sc			
				Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
				Inner join @upermitted up on scc.IdUser = up.Id
			Where sc.RecordAccess = 2 --Участники слушают
			Group By sc.Id

		Insert into #A_Stat_Conference_tmp2
			Select sc.Id 
			From #A_Stat_Conference_tmp sc
				Inner join A_Stat_Conference_RecordAccessUsers scr on sc.Id = scr.IdConf
				Inner join @upermitted up on scr.IdUser = up.Id
			Where sc.RecordAccess = 3 --Указанные участники слушают
			Group By sc.Id
			
		-----------------------------
		-- права на внешние линии ---
		-----------------------------

		if ( @canlistenext > 0 ) begin
			-- Тот, у кого есть право на все внешние линии, слушает
			Insert into #A_Stat_Conference_tmp2
				Select sc.Id 
				From #A_Stat_Conference_tmp sc
					Inner join A_Stat_Conference_Competitors scc on scc.IdConf = sc.Id
				Where scc.IdUser = 'BF000000-0000-0000-0000-000000000000'
					And not exists ( Select id from #A_Stat_Conference_tmp2 where id = sc.Id )
				Group By sc.Id
		end
	    
		-----------------------------
		-- Права на отдельно-взятые линии (через направления в т.ч.)
		-----------------------------
	    
		Insert into #A_Stat_Conference_tmp2
			Select sc.Id 
			From #A_Stat_Conference_tmp sc
				Inner join A_Stat_Conference_Competitors scc on scc.IdConf = sc.Id
				Inner join @lpermitted l on scc.IdLine = l.Id
			Where not exists ( Select id from #A_Stat_Conference_tmp2 where id = sc.Id )
			Group By sc.Id	
		
		-----------------------------
		-- По хештегам (цепочки, конференции и их коммутации)
		-----------------------------

		-----------------------------------------------
		-- Внутренние номера (постфильтр) -------------
		-----------------------------------------------

		if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp3' ))
			Drop table #A_Stat_Conference_tmp3
		Create table #A_Stat_Conference_tmp3 ( IdItem uniqueidentifier PRIMARY KEY )

		if ( @innerprefix is not null ) begin

			Insert into #A_Stat_Conference_tmp3 ( iditem )
				Select ReactId
				From @intnumbers t
				Where ( prefix like @innerprefix )
					and ( addrtype in ( 0, 1 ))
		end

    end

--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
-----------------------------------------------------------
------- Окончательная выборка -----------------------------
-----------------------------------------------------------

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Result' ))
		Drop table #A_Stat_Result

	--------------------------------
	-- Коммутации
	--------------------------------
	Select sc.*, 
		0 as IsConf, @idempty as Director, sc.TimeStart as CTS, 
		case When ( c.Id is null ) then 0 else 1 end as HasComment
	Into #A_Stat_Result
	From A_Stat_Connections_1x1 sc
		Inner join #A_Stat_Connections_1x1_tmp sct on sc.Id=sct.Id
		Left join A_Stat_Comments c on sc.Id = c.Id

	--------------------------------
	-- Неудачные исходящие
	--------------------------------

   	if (( @showfailed = 1 ) and ( @lt = 0 )) begin

	    Insert into #A_Stat_Result ( Id, IdChain, IdPrev, IdNext,
					TimeStart, TimeAnswer, TimeStop,
					ReasonStart, ReasonStop, StopSide,
					ALineId, BLineId,
					AOutNumber, BOutNumber, 
					ALineNum, BLineNum,
					AUserId, BUserId,
					ANumberDialed, BNumberDialed,
					IsRecorded, ConnectionType,
					AStr, BStr,
					FromCC, IsBetweenUsers, IdRecDir,
					IsConf, Director, CTS, HasComment )

 	    	Select sc.Id, sc.IdChain, @idempty as IdPrev, @idempty as IdNext, 
			sc.TimeStart, sc.TimeStop as TimeAnswer, sc.TimeStop,
			sc.ReasonStart, sc.ReasonFailed, 0 as StopSide, 
			sc.ALineId, sc.BLineId,
			sc.AOutNumber, sc.ANumberDialed as BOutNumber,
			sc.ALineNum, sc.BLineNum,
			sc.AUserId, @idempty as BUserId,
			sc.ANumberDialed, '' as BNumberDialed,
			0 as IsRecorded, 
			21 as ConnectionType,
			sc.AStr, sc.BStr, 
			sc.FromCC,
			case when ( sc.BLineId is NULL or sc.BLineId = @idempty ) then 1 else 0 end as IsBetweenUsers,
			0 as IdRecDir,
			0 as IsConf, @idempty as Director, sc.TimeStart as CTS,
			case When ( c.Id is null ) then 0 else 1 end as HasComment
 
		From A_Stat_FailedCalls sc
			Inner join #A_Stat_Failed_tmp sct on sc.Id = sct.Id
			Left join A_Stat_Comments c on sc.Id = c.Id
	end

	------------------------------------------------
	---- Конференции --------------------------------
	------------------------------------------------

    	if ( @showconference = 1 ) begin

			Insert into #A_Stat_Result ( Id, IdChain, IdPrev, IdNext,
						TimeStart, TimeAnswer, TimeStop,
						ReasonStart, ReasonStop, StopSide,
						ALineId, BLineId,
						AOutNumber, BOutNumber, 
						ALineNum, BLineNum,
						AUserId, BUserId,
						ANumberDialed, BNumberDialed,
						IsRecorded, ConnectionType,
						AStr, BStr,
						FromCC, IsBetweenUsers, IdRecDir,
						IsConf, Director, CTS, HasComment )

   	    		-- Ид конференции пусть выступает цепочкой, по ней можно будет группировать конференции в статистике
	    		-- А ид коммутации так и остается ид коммутации
	    		Select IdConn, sc.Id as IdChain, @idempty as IdPrev, @idempty as IdNext, 
					scc.TimeStart, scc.TimeStart as TimeAnswer, scc.TimeStop, 
					ReasonStart, ReasonStop, 0 as StopSide, 
					IdLine as ALineId, IdConf as BLineId, 
					OutNumber as AOutNumber, '' as BOutNumber, 
					LineNum as ALineNum, SubString ( cast ( Room as nvarchar ( 10 ) ), 1, 10 ) as BLineNum, 
					IdUser as AUserId, @idconfuser as BUserId, 
					'' as ANumberDialed, '' as BNumberDialed, 
					IsRecorded, case when ( sc.Type = 1 ) then 11/*селектор*/ else 10/*конф*/ end as ConnectionType, 
					StrName as AStr, SubString ( case when ( Name != '' ) then Name else '#' + cast ( Room as nvarchar ( 10 ) ) end, 1, 200 ) as BStr, 
					0 as FromCC, 1 as IsBetweenUsers, IdRecDir, 
					1 as IsConf, sc.Director, sc.TimeStart as CTS,
					case When ( c.Id is null ) then 0 else 1 end as HasComment

				From A_Stat_Conference_Competitors scc
					Inner join A_Stat_Conference sc on sc.Id = scc.IdConf
					Inner join #A_Stat_Conference_tmp2 sct on sc.Id = sct.Id --фильтр по правам
					Left join A_Stat_Comments c on sc.Id = c.Id
					--Inner join @upermitted up on up.Id = scc.IdUser -- чтобы в конференции даже если она разрешена на просмотр не видеть записи неподчиненных

				Where ((( scc.TimeStart <= @dstart ) and ( scc.TimeStop > @dstart ))
					or (( scc.TimeStart < @dstop ) and ( scc.TimeStop >= @dstop ))
					or (( scc.TimeStart <= @dstart ) and ( scc.TimeStop > @dstart ))
					or (( scc.TimeStart >= @dstart ) and ( scc.TimeStop <= @dstop )))
					and ( Convert ( float, scc.TimeStop ) - Convert ( float, scc.TimeStart ) >= @lt )
					and (( @operatorid is null ) or ( scc.IdUser = @operatorid ))
					and (( @OutNumber is null ) or ( scc.OutNumber = @OutNumber ))
					and (( @innerprefix is null ) 
						or ( scc.IdLine in (Select iditem from #A_Stat_Conference_tmp3)) 
						or ( scc.IdUser in (Select iditem from #A_Stat_Conference_tmp3 )))
    	end

	
    ------------------------------------------------
    ---- Финальная выборка -------------------------
    ------------------------------------------------

	Select * 
	From #A_Stat_Result

end
GO