﻿if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Personal_ListInfo_Delete' ))
	Drop procedure dbo.A_Personal_ListInfo_Delete
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Personal_ListInfo_Save' ))
	Drop procedure dbo.A_Personal_ListInfo_Save
GO

Create procedure [dbo].[A_Personal_ListInfo_Save]
	@userid uniqueidentifier,
	@tab int,
	@view int,	
	@fullclear bit,
	@xml ntext
as
begin

	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		tabId int, 
		viewId int, 
		numId uniqueidentifier, 
		visible bit, 
		x int, 
		y int, 
		selfdescription nvarchar(256) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/a', 2)	
		With ( 			
			tabId int '@tab',
			viewId int '@view',
			numId varchar(50) '@num',
			visible bit '@vis',
			x int '@x',
			y int '@y',
			descr nvarchar(256) '@descr'
		)	
		

	if ( @fullclear = 1 ) begin
		
		Delete 
		From dbo.A_Personal_ListInfo 
		Where (UserId = @userid)
			and (TabId = @tab)
			and (ViewId = @view)			
		
	end else begin

		Delete
		From dbo.A_Personal_ListInfo
		Where UserId = @userid
			and exists ( Select 1
						 From #tmp_xml x 
						 Where (dbo.A_Personal_ListInfo.TabId = x.tabId) 
							and (dbo.A_Personal_ListInfo.ViewId = x.viewId) 
							and (dbo.A_Personal_ListInfo.NumberId = x.numId) )
	
	end
	
	Insert into dbo.A_Personal_ListInfo (Id, UserId, TabId, ViewId, NumberId, Visible, X, Y, SelfDescription)
		Select NewId(), @userid, tabId, viewId, numId, visible, x, y, selfdescription
		From #tmp_xml x
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_VoiceMail_SaveData' ))
	Drop procedure dbo.A_VoiceMail_SaveData
GO

Create procedure [dbo].[A_VoiceMail_SaveData]
	@id uniqueidentifier,
	@recieverid uniqueidentifier,
	@recieverisgroup int,
	@senttime datetime,
	@filepath nvarchar ( 500 ),
	@info nvarchar ( 500 ),
	@callerid nvarchar ( 50 ),
	@isshown int,
	@whorecieved uniqueidentifier,
	@receiverxml ntext
as
begin

	if ( @isshown > 0 )	begin
		
		Insert Into A_VoiceMail ( Id, RecieverId, RecieverIsGroup, SentTime, RecievedTime, FilePath, Info, CallerId, IsShown, WhoRecieved )
		Values ( @id, @recieverid, @recieverisgroup, @senttime, @senttime, @filepath, @info, @callerid, @isshown, @whorecieved )
			
	end	else begin
		
		Insert Into A_VoiceMail ( Id, RecieverId, RecieverIsGroup, SentTime, FilePath, Info, CallerId, IsShown )
		Values ( @id, @recieverid, @recieverisgroup, @senttime, @filepath, @info, @callerid, @isshown )

	end
	
	-- Получатели (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( userid uniqueidentifier )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @receiverxml
	
	Insert into #tmp_xml
		Select userid
		From OpenXml (@idoc, '/items/a', 2)	
		With ( userid varchar(50) '@uid' )	
	
	
	Insert into A_VoiceMailReceivers ( msgid, userid )
		Select @id, userid
		From #tmp_xml	
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_DB_UpdateTransactionLog' ))
	Drop procedure dbo.A_DB_UpdateTransactionLog
GO

Create procedure dbo.A_DB_UpdateTransactionLog
	@dbname nvarchar(200) = NULL,
	@size int = 500,
	@modifymaxsize bit = 1	
as
begin

	-- DB name
	if (@dbname is NULL)
		Set @dbname = DB_name()
	
	-- Product version
	Declare @vrs nvarchar(50), @major int
	Select @vrs = cast ( ServerProperty ( 'ProductVersion' ) as nvarchar(50))
	Select @major = Cast ( Left(@vrs, PatIndex('%.%', @vrs) - 1) as int)

	-- Auto shrink, auto close
	if exists ( Select * FROM master..sysdatabases where name like @dbname ) begin
		
		if ( @major > 11 ) begin
		
			Exec ( N'Alter database [' + @dbname + '] set AUTO_CLOSE off' )
			Exec ( N'Alter database [' + @dbname + '] set AUTO_SHRINK on' )
			
		end else begin
		
			Exec sp_dboption @dbname, N'autoclose', N'false'
			Exec sp_dboption @dbname, N'autoshrink', N'true'
			
		end
	end
	
	-- Transaction log file size
	if exists ( Select * FROM Master..sysdatabases where name like @dbname ) begin
		
		Declare @sql nvarchar(4000)
		
		Set @sql = '
			Declare @logname nvarchar(100)
			Declare @sz int, @msz int
			
			Select 
				@logname = name,
				@sz = size, 
				@msz = maxsize 
			From [' + @dbname + '].dbo.sysfiles 
			Where (status & 0x40) != 0			
		'
		
		Set @sql = @sql + '				
			if ( @sz < ' + cast ( @size as nvarchar(20)) + ' * 128 )
				Exec ( N''Alter database [' + @dbname + '] modify file ( name='' + @logname + '', size=' + cast (@size as nvarchar(20)) + ')'')
		'
		
		if ( @modifymaxsize = 1 )		
			Set @sql = @sql + '
			if ( @msz < 1500 * 128 )
				Exec ( N''Alter database [' + @dbname + '] modify file ( name='' + @logname + '', maxsize=1500)'')
			'
		
		Set @sql = @sql + '			
			Exec ( N''Alter database [' + @dbname + '] set recovery simple'' )
		'
				
		Exec sp_executesql @sql
		
	end
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_DB_CreateBackup' ))
	Drop procedure dbo.A_DB_CreateBackup
GO

Create procedure dbo.A_DB_CreateBackup
	@dbname nvarchar(200),
	@path nvarchar(4000)
as
begin
		
	if exists ( Select * FROM Master..sysdatabases where name like @dbname ) begin
	
		Declare @sql nvarchar(4000)
		Set @sql = '
			Use master
			
			Backup Database [' + @dbname + ']
			TO DISK = N''' + @path + '''
			With INIT, SKIP
		'
		
		exec sp_executesql @sql
		
	end
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Settings_License_GetAgreementTick' ))
	Drop procedure dbo.A_Settings_License_GetAgreementTick
GO

Create procedure dbo.A_Settings_License_GetAgreementTick
as
begin
	
	Declare @v1 int
	Select @v1 = ValueInt From A_Settings Where name like 'Agreement'
	Select @v1 as AgreementTick

end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Settings_License_SetAgreementTick' ))
	Drop procedure dbo.A_Settings_License_SetAgreementTick
GO

Create procedure dbo.A_Settings_License_SetAgreementTick
	@val int
as
begin
	
	if exists ( Select 1 From A_Settings where name like 'Agreement' ) begin	
	
		Update A_Settings
		Set ValueInt = @val
		Where (name like 'Agreement')
			and (ValueInt != @val)

	end else begin
	
		Insert into A_Settings ( [Name], [ValueInt] )
		Values ( 'Agreement', @val )
		
	end

end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_Workplaces_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_Workplaces_Save
GO

Create procedure dbo.A_Admin_OfficeMap_Workplaces_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Рабочие места (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	(
		[id] uniqueidentifier primary key, 
		[type] int, 
		[parentid] uniqueidentifier, 
		[address] nvarchar(500) collate database_default, 
		[hdb] bit, 
		[hfe] bit, 
		[hl] bit, 
		[hel] bit, 
		[hil] bit, 
		[hgsm] bit, 
		[name] nvarchar(500), 
		[lineid] uniqueidentifier collate database_default, 
		[sort] int
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/wp', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			parentid varchar(50) '@par',
			address nvarchar(500) '@addr',
			hdb bit '@hdb',
			hfe bit '@hfe',
			hl bit '@hl',
			hel bit '@hel',
			hil bit '@hil',
			hgsm bit '@hgsm',
			name nvarchar(500) '@name',
			lineid varchar(50) '@lid',
			sort int '@sort'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_Workplaces
	Where not exists ( Select 1 From #tmp_xml x Where A_Workplaces.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ParentId = x.parentid,
		w.Address = x.address,
		w.HasDatabase = x.hdb,
		w.HasExtLines = x.hel,
		w.HasIntLines = x.hil,
		w.HasGsm = x.hgsm,
		w.HasFastEngine = x.hfe,
		w.HasLogic = x.hl,
		w.Name = x.name,
		w.LineId = x.lineid,
		w.SortIndex = x.sort		
	From dbo.A_Workplaces w, #tmp_xml x
	Where (w.Id = x.id)
		and ((w.Type != x.type)
			or (w.ParentId != x.parentid)
			or (w.Address != x.address)
			or (w.HasDatabase != x.hdb)
			or (w.HasExtLines != x.hel)
			or (w.HasIntLines != x.hil)
			or (w.HasGsm != x.hgsm)
			or (w.HasFastEngine != x.hfe)
			or (w.HasLogic != x.hl)
			or (w.Name != x.name)
			or (w.LineId != x.lineid)
			or (w.SortIndex != x.sort))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Workplaces (Id, Type, ParentId, Address, HasDatabase, HasExtLines, HasIntLines, HasGsm, HasFastEngine, HasLogic, Name, LineId, SortIndex)
		Select Distinct id, type, parentid, address, hdb, hfe, hl, hel, hil, hgsm, name, lineid, sort
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_Workplaces w Where w.Id = x.id )
			
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerNumber_Clear' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerNumber_Clear
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerNumber_Set' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerNumber_Set
GO		
		
----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerNumbers_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerNumbers_Save
GO

Create procedure dbo.A_Admin_OfficeMap_ServerNumbers_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Номера серверов (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		num int 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, num
		From OpenXml (@idoc, '/items/srv', 2)	
		With ( 
			id varchar(50) '@id',
			num int '@num'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerNumbers
	Where not exists ( Select 1 From #tmp_xml x Where A_ServerNumbers.ID = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.PrefNum = x.num
	From dbo.A_ServerNumbers w, #tmp_xml x
	Where (w.ID = x.id)
		and (w.PrefNum != x.num)
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerNumbers (ID, PrefNum)
		Select Distinct id, num
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerNumbers w Where w.ID = x.id )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerGate_Clear' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerGate_Clear
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerGate_Set' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerGate_Set
GO		
		
----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerGates_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerGates_Save
GO

Create procedure dbo.A_Admin_OfficeMap_ServerGates_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Шлюзы (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		name nvarchar(500) collate database_default, 
		regtype int, 
		srvid uniqueidentifier 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, name, regtype, srvid
		From OpenXml (@idoc, '/items/gate', 2)	
		With ( 
			id varchar(50) '@id',
			name nvarchar(500) '@name',
			regtype int '@regtype',
			srvid varchar(50) '@srvid'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerGates
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerGates.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IdServer = x.srvid,
		w.RegType = x.regtype
	From dbo.A_ServerGates w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Name != x.name)
			or (w.IdServer != x.srvid)
			or (w.RegType != x.regtype))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerGates (Id, Name, IdServer, RegType)
		Select Distinct id, name, srvid, regtype
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerGates w Where w.Id = x.id )
		
end
GO		

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerGate_Get' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerGate_Get
GO

Create procedure [dbo].[A_Admin_OfficeMap_ServerGate_Get]	
	@idserver uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin

	if ( @idserver = '00000000-0000-0000-0000-000000000000' )

		Select sg.Id, sg.Name, sg.IdServer, sg.RegType
		From A_ServerGates sg

	else

		Select sg.Id, sg.Name, sg.IdServer, sg.RegType
		From A_ServerGates sg
		Where sg.IdServer = @idserver
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerStream_Clear' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerStream_Clear
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerStream_Set' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerStream_Set
GO		
		
----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerStreams_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_ServerStreams_Save
GO

Create procedure dbo.A_Admin_OfficeMap_ServerStreams_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		name nvarchar(500) collate database_default, 
		type int, 
		srvid uniqueidentifier, 
		sipuser nvarchar(500) collate database_default, 
		regtype int, 
		gateid uniqueidentifier, 
		dialmode int 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/stream', 2)	
		With ( 
			id varchar(50) '@id',
			name nvarchar(500) '@name',
			type int '@type',
			srvid varchar(50) '@srvid',
			sipuser nvarchar(500) '@sipuser',
			regtype int '@regtype',
			gateid varchar(50) '@gate',
			dialmode int '@dialmode'
		)	
		
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerStreams
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerStreams.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IdServer = x.srvid,
		w.Type = x.type,
		w.Account = x.sipuser,
		w.RegType = x.regtype,
		w.GateId = x.gateid,
		w.DialMode = x.dialmode
	From dbo.A_ServerStreams w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Name != x.name)
			or (w.IdServer != x.srvid)
			or (w.Type != x.type)
			or (w.Account != x.sipuser)
			or (w.RegType != x.regtype)
			or (w.GateId != x.gateid)
			or (w.DialMode != x.dialmode))			
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerStreams (Id, Name, Type, IdServer, Account, RegType, GateId, DialMode)
		Select Distinct id, name, type, srvid, sipuser, regtype, gateid, dialmode 
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerStreams w Where w.Id = x.id )
		
end
GO		


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ExtLines_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_ExtLines_Save
GO

Create procedure dbo.A_Admin_OfficeMap_ExtLines_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		type int,
		serverid uniqueidentifier,
		caption nvarchar(50) collate database_default,
		dialtype int,
		cansimpleout bit, 
		canip bit,
		canautodial bit,
		prefix nvarchar(50) collate database_default,
		systemnum int,
		systemnumstr nvarchar(10) collate database_default,
		enabled bit,
		canmobile bit,
		extnumber nvarchar(20) collate database_default,
		connectedpbx bit,
		streamid uniqueidentifier,
		linecode nvarchar(200) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/el', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			serverid varchar(50) '@srvid', 
			caption nvarchar(200) '@cap',
			dialtype int '@dial',
			cansimpleout bit '@cso',
			canip bit '@cip',
			canautodial bit '@cad',
			prefix nvarchar(50) '@pref',
			systemnum int '@snum',
			systemnumstr nvarchar(10) '@snums',
			enabled bit '@en',
			canmobile bit '@cm',
			extnumber nvarchar(20) '@exnum',
			connectedpbx bit '@pbx',
			streamid varchar(50) '@stream',
			linecode nvarchar(200) '@lc'			
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerExtLines
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerExtLines.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ServerId = x.serverid,
		w.Caption = x.caption,
		w.DialType = x.dialtype,
		w.CanSimpleOut = x.cansimpleout,
		w.CanIp = x.canip,
		w.CanAutoDial = x.canautodial,
		w.Prefix = x.prefix,
		w.SystemNum = x.systemnum,
		w.SystemNumStr = x.systemnumstr,
		w.Enabled = x.enabled,
		w.CanMobile = x.canmobile,
		w.ExtNumber = x.extnumber,
		w.ConnectedPBX = x.connectedpbx,
		w.StreamId = x.streamid,
		w.LineCode = x.linecode
	From dbo.A_ServerExtLines w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Type != x.type)
			or (w.ServerId != x.serverid)
			or (w.Caption != x.caption)
			or (w.DialType != x.dialtype)
			or (w.CanSimpleOut != x.cansimpleout)
			or (w.CanIp != x.canip)
			or (w.CanAutoDial != x.canautodial)
			or (w.Prefix != x.prefix)
			or (w.SystemNum != x.systemnum)
			or (w.SystemNumStr != x.systemnumstr)
			or (w.Enabled != x.enabled)
			or (w.CanMobile != x.canmobile)
			or (w.ExtNumber != x.extnumber)
			or (w.ConnectedPBX != x.connectedpbx)
			or (w.StreamId != x.streamid)
			or (w.LineCode != x.linecode))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerExtLines (Id, Type, ServerId, Caption, DialType, CanSimpleOut, CanIp, CanAutoDial, Prefix,
			SystemNum, SystemNumStr, Enabled, CanMobile, ExtNumber, ConnectedPBX, StreamId, LineCode)
		Select Distinct id, type, serverid, caption, dialtype, cansimpleout, canip, canautodial, prefix,
			systemnum, systemnumstr, enabled, canmobile, extnumber, connectedpbx, streamid, linecode
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerExtLines w Where w.Id = x.id )
		
end
GO		


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_IntLines_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_IntLines_Save
GO

Create procedure dbo.A_Admin_OfficeMap_IntLines_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		type int,
		serverid uniqueidentifier,
		caption nvarchar(50) collate database_default,
		systemnum int,
		systemnumstr nvarchar(10) collate database_default,
		enabled bit,
		citystraight bit,
		parentgroupid uniqueidentifier,
		defaultuserid uniqueidentifier,
		websockuserid uniqueidentifier,
		linecode nvarchar(200) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/il', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			serverid varchar(50) '@srvid', 
			caption nvarchar(200) '@cap',
			systemnum int '@snum',
			systemnumstr nvarchar(10) '@snums',
			enabled bit '@en',
			citystraight bit '@city',
			parentgroupid varchar(50) '@parid',
			defaultuserid varchar(50) '@defid',
			websockuserid varchar(50) '@wsid',
			linecode nvarchar(200) '@lc'			
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerIntLines
	Where not exists ( Select 1 From #tmp_xml x Where A_ServerIntLines.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ServerId = x.serverid,
		w.Caption = x.caption,
		w.SystemNum = x.systemnum,
		w.SystemNumStr = x.systemnumstr,
		w.Enabled = x.enabled,
		w.CityStraight = x.citystraight,
		w.ParentGroupId = x.parentgroupid,
		w.DefaultUserId = x.defaultuserid,
		w.WebsockUserId = x.websockuserid,
		w.LineCode = x.linecode
	From dbo.A_ServerIntLines w, #tmp_xml x
	Where (w.Id = x.id)
		and ((w.Type != x.type)
			or (w.ServerId != x.serverid)
			or (w.Caption != x.caption)
			or (w.SystemNum != x.systemnum)
			or (w.SystemNumStr != x.systemnumstr)
			or (w.Enabled != x.enabled)
			or (w.CityStraight != x.citystraight)
			or (w.ParentGroupId != x.parentgroupid)
			or (w.DefaultUserId != x.defaultuserid)
			or (w.WebsockUserid != x.websockuserid)
			or (w.LineCode != x.linecode))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerIntLines (Id, Type, ServerId, Caption, SystemNum, SystemNumStr, Enabled, 
			CityStraight, ParentGroupId, DefaultUserId, WebsockUserId, LineCode)
		Select Distinct id, type, serverid, caption, systemnum, systemnumstr, enabled, 
			citystraight, parentgroupid, defaultuserid, websockuserid, linecode
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerIntLines w Where w.Id = x.id )
		
end
GO		


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_UsbLines_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_UsbLines_Save
GO

Create procedure dbo.A_Admin_OfficeMap_UsbLines_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		type int,
		serverid uniqueidentifier,
		caption nvarchar(50) collate database_default,
		systemnum int,
		systemnumstr nvarchar(10) collate database_default,
		enabled bit,
		citystraight bit,
		parentgroupid uniqueidentifier,
		linecode nvarchar(200) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/il', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			serverid varchar(50) '@srvid', 
			caption nvarchar(200) '@cap',
			systemnum int '@snum',
			systemnumstr nvarchar(10) '@snums',
			enabled bit '@en',
			citystraight bit '@city',
			parentgroupid varchar(50) '@parid',
			linecode nvarchar(200) '@lc'			
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerUsbLines
	Where not exists ( Select 1 From #tmp_xml x Where A_ServerUsbLines.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ServerId = x.serverid,
		w.Caption = x.caption,
		w.SystemNum = x.systemnum,
		w.SystemNumStr = x.systemnumstr,
		w.Enabled = x.enabled,
		w.CityStraight = x.citystraight,
		w.ParentGroupId = x.parentgroupid,
		w.LineCode = x.linecode
	From dbo.A_ServerUsbLines w, #tmp_xml x
	Where (w.Id = x.id)
		and ((w.Type != x.type)
			or (w.ServerId != x.serverid)
			or (w.Caption != x.caption)
			or (w.SystemNum != x.systemnum)
			or (w.SystemNumStr != x.systemnumstr)
			or (w.Enabled != x.enabled)
			or (w.CityStraight != x.citystraight)
			or (w.ParentGroupId != x.parentgroupid)
			or (w.LineCode != x.linecode))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerUsbLines (Id, Type, ServerId, Caption, SystemNum, SystemNumStr, Enabled, 
			CityStraight, ParentGroupId, LineCode)
		Select Distinct id, type, serverid, caption, systemnum, systemnumstr, enabled, 
			citystraight, parentgroupid, linecode
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerUsbLines w Where w.Id = x.id )
		
end
GO		


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_SipLines_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_SipLines_Save
GO

Create procedure dbo.A_Admin_OfficeMap_SipLines_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		type int,
		serverid uniqueidentifier,
		caption nvarchar(50) collate database_default,
		systemnum int,
		systemnumstr nvarchar(10) collate database_default,
		enabled bit,
		citystraight bit,
		parentgroupid uniqueidentifier,
		defaultuserid uniqueidentifier,
		websockuserid uniqueidentifier,
		linecode nvarchar(200) collate database_default,
		address nvarchar(500) collate database_default,
		streamid uniqueidentifier,
		busymode int,		
		translit int,
		sippass nvarchar(100) collate database_default,
		siplogin nvarchar(100) collate database_default,
		sipuser nvarchar(100) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/il', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			serverid varchar(50) '@srvid', 
			caption nvarchar(200) '@cap',
			systemnum int '@snum',
			systemnumstr nvarchar(10) '@snums',
			enabled bit '@en',
			citystraight bit '@city',
			parentgroupid varchar(50) '@parid',
			defaultuserid varchar(50) '@defid',
			websockuserid varchar(50) '@wsid',
			linecode nvarchar(200) '@lc',
			address nvarchar(500) '@addr',
			streamid uniqueidentifier '@stream',
			busymode int '@busy',
			translit int '@transl',
			sippass nvarchar(100) '@sippass',
			siplogin nvarchar(100) '@siplogin',
			sipuser nvarchar(100) '@sipuser'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerSipLines		
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerSipLines.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ServerId = x.serverid,
		w.Caption = x.caption,
		w.SystemNum = x.systemnum,
		w.SystemNumStr = x.systemnumstr,
		w.Enabled = x.enabled,
		w.CityStraight = x.citystraight,
		w.ParentGroupId = x.parentgroupid,
		w.DefaultUserId = x.defaultuserid,
		w.WebsockUserId = x.websockuserid,
		w.LineCode = x.linecode,
		w.Address = x.address,
		w.StreamId = x.streamid,
		w.BusyMode = x.busymode,
		w.Translit = x.translit,
		w.SipPass = x.sippass,
		w.SipLogin = x.siplogin,
		w.SipUser = x.sipuser		
	From dbo.A_ServerSipLines w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Type != x.type)
			or (w.ServerId != x.serverid)
			or (w.Caption != x.caption)
			or (w.SystemNum != x.systemnum)
			or (w.SystemNumStr != x.systemnumstr)
			or (w.Enabled != x.enabled)
			or (w.CityStraight != x.citystraight)
			or (w.ParentGroupId != x.parentgroupid)
			or (w.DefaultUserId != x.defaultuserid)
			or (w.WebsockUserid != x.websockuserid)
			or (w.LineCode != x.linecode)
			or (w.Address != x.address)
			or (w.StreamId != x.streamid)
			or (w.BusyMode != x.busymode)
			or (w.Translit != x.translit)
			or (w.SipPass != x.sippass)
			or (w.SipLogin != x.siplogin)
			or (w.SipUser != x.sipuser))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerSipLines (Id, Type, ServerId, Caption, SystemNum, SystemNumStr, Enabled, 
			CityStraight, ParentGroupId, DefaultUserId, WebsockUserId, LineCode,
			Address, StreamId, BusyMode, Translit, SipPass, SipLogin, SipUser)
		Select Distinct id, type, serverid, caption, systemnum, systemnumstr, enabled, 
			citystraight, parentgroupid, defaultuserid, websockuserid, linecode,
			address, streamid, busymode, translit, sippass, siplogin, sipuser
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerSipLines w Where w.Id = x.id )
		
end
GO		

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_RemoveTails' ))
	Drop procedure dbo.A_Admin_OfficeMap_RemoveTails
GO

Create Procedure [dbo].[A_Admin_OfficeMap_RemoveTails]
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Delete from A_LinesReactors

	Insert Into A_LinesReactors
	Select wp.LineID, wp.Type, wp.ParentID FROM A_WorkPlaces wp
		inner join A_WorkPlaces wp2 on wp.ParentID = wp2.ID
		Where wp.LineID != '00000000-0000-0000-0000-000000000000'
		And wp2.Type != 0

	Delete from A_WorkPlace_Address

	exec A_Admin_UserGroup_Group_SiftEmpty
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_VirtualLine_Gate_ClearAll' ))
	Drop procedure dbo.A_Admin_VirtualLine_Gate_ClearAll
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_VirtualLine_Gate_Insert' ))
	Drop procedure dbo.A_Admin_VirtualLine_Gate_Insert
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_VirtualLine_Gate_GetAll' ))
	Drop procedure dbo.A_Admin_VirtualLine_Gate_GetAll
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearExtLines' ))
	Drop procedure dbo.A_TaskManager_Task_ClearExtLines
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ExtlinesSave' ))
	Drop procedure dbo.A_TaskManager_Task_ExtlinesSave
GO

Create procedure dbo.A_TaskManager_Task_ExtlinesSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier primary key	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id
		From OpenXml (@idoc, '/items/l', 2)	
		With ( id varchar(50) '@id' )	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_TaskExtLines		
	Where (TaskId = @taskid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where A_TaskManager_TaskExtLines.ExtLineId = x.id )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_TaskExtLines (TaskId, ExtLineId)
		Select Distinct @taskid, x.id
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_TaskExtLines w 
							Where (w.TaskId = @taskid) 
								and (w.ExtLineId = x.id) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearOperators' ))
	Drop procedure dbo.A_TaskManager_Task_ClearOperators
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_OperatorsSave' ))
	Drop procedure dbo.A_TaskManager_Task_OperatorsSave
GO

Create procedure dbo.A_TaskManager_Task_OperatorsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier primary key, isgroup bit, priority int )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, isgroup, priority
		From OpenXml (@idoc, '/items/op', 2)	
		With ( 
			id varchar(50) '@id',
			isgroup bit '@isgr',
			priority int '@pr'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_Operators
	Where (TaskId = @taskid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where A_TaskManager_Operators.OperatorId = x.id )
		
	-- Обновляем тех, у которых что-то изменилось
	Update w
	Set w.IsGroup = x.isgroup,
		w.Priority = x.priority		
	From dbo.A_TaskManager_Operators w, #tmp_xml x
	Where (w.TaskId = @taskid)
		and (w.OperatorId = x.id)
		and ((w.IsGroup != x.isgroup)
			or (w.Priority != x.priority))
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_Operators (TaskId, OperatorId, IsGroup, Priority)
		Select Distinct @taskid, x.id, x.isgroup, x.priority
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_Operators w 
							Where (w.TaskId = @taskid) 
								and (w.OperatorId = x.id) )
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearSupervisors' ))
	Drop procedure dbo.A_TaskManager_Task_ClearSupervisors
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_SupervisorsSave' ))
	Drop procedure dbo.A_TaskManager_Task_SupervisorsSave
GO

Create procedure dbo.A_TaskManager_Task_SupervisorsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier primary key )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id
		From OpenXml (@idoc, '/items/spv', 2)	
		With ( id varchar(50) '@id' )	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_TaskSupervisors
	Where (TaskId = @taskid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where A_TaskManager_TaskSupervisors.SupervisorId = x.id )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_TaskSupervisors (TaskId, SupervisorId)
		Select Distinct @taskid, x.id
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_TaskSupervisors w 
							Where (w.TaskId = @taskid) 
								and (w.SupervisorId = x.id) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearPeriods' ))
	Drop procedure dbo.A_TaskManager_Task_ClearPeriods
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_PeriodsSave' ))
	Drop procedure dbo.A_TaskManager_Task_PeriodsSave
GO

Create procedure dbo.A_TaskManager_Task_PeriodsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( dow int, start datetime, fin datetime )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select dow, start, fin
		From OpenXml (@idoc, '/items/p', 2)	
		With ( 
			dow int '@dow',
			start datetime '@start',
			fin datetime '@fin'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_RunPeriods
	Where (TaskId = @taskid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where (A_TaskManager_RunPeriods.DayOfWeek = x.dow)
							and (A_TaskManager_RunPeriods.StartTime = x.start) 
							and (A_TaskManager_RunPeriods.FinishTime = x.fin) )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_RunPeriods (TaskId, DayOfWeek, StartTime, FinishTime)
		Select Distinct @taskid, x.dow, x.start, x.fin
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_RunPeriods w 
							Where (w.TaskId = @taskid) 
								and (w.DayOfWeek = x.dow) 
								and (w.StartTime = x.start) 
								and (w.FinishTime = x.fin) )
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearOperators' ))
	Drop procedure dbo.A_SmsTaskManager_Task_ClearOperators
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_OperatorsSave' ))
	Drop procedure dbo.A_SmsTaskManager_Task_OperatorsSave
GO

Create procedure dbo.A_SmsTaskManager_Task_OperatorsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier primary key, isgroup bit, priority int )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, isgroup, priority
		From OpenXml (@idoc, '/items/op', 2)	
		With ( 
			id varchar(50) '@id',
			isgroup bit '@isgr',
			priority int '@pr'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_SmsTaskManager_Operators
	Where (TaskId = @taskid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where A_SmsTaskManager_Operators.OperatorId = x.id )
		
	-- Обновляем тех, у которых что-то изменилось
	Update w
	Set w.IsGroup = x.isgroup,
		w.Priority = x.priority		
	From dbo.A_SmsTaskManager_Operators w, #tmp_xml x
	Where (w.TaskId = @taskid)
		and (w.OperatorId = x.id)
		and ((w.IsGroup != x.isgroup)
			or (w.Priority != x.priority))
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_SmsTaskManager_Operators (TaskId, OperatorId, IsGroup, Priority)
		Select Distinct @taskid, x.id, x.isgroup, x.priority
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_SmsTaskManager_Operators w 
							Where (w.TaskId = @taskid) 
								and (w.OperatorId = x.id) )
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearSupervisors' ))
	Drop procedure dbo.A_SmsTaskManager_Task_ClearSupervisors
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_SupervisorsSave' ))
	Drop procedure dbo.A_SmsTaskManager_Task_SupervisorsSave
GO

Create procedure dbo.A_SmsTaskManager_Task_SupervisorsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier primary key )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id
		From OpenXml (@idoc, '/items/spv', 2)	
		With ( id varchar(50) '@id' )	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_SmsTaskManager_Supervisors
	Where (TaskId = @taskid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where A_SmsTaskManager_Supervisors.SupervisorId = x.id )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_SmsTaskManager_Supervisors (TaskId, SupervisorId)
		Select Distinct @taskid, x.id
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_SmsTaskManager_Supervisors w 
							Where (w.TaskId = @taskid) 
								and (w.SupervisorId = x.id) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearPeriods' ))
	Drop procedure dbo.A_SmsTaskManager_Task_ClearPeriods
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_PeriodsSave' ))
	Drop procedure dbo.A_SmsTaskManager_Task_PeriodsSave
GO

Create procedure dbo.A_SmsTaskManager_Task_PeriodsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( dow int, start datetime, fin datetime )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select dow, start, fin
		From OpenXml (@idoc, '/items/p', 2)	
		With ( 
			dow int '@dow',
			start datetime '@start',
			fin datetime '@fin'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_SmsTaskManager_RunPeriods
	Where (TaskId = @taskid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where (A_SmsTaskManager_RunPeriods.DayOfWeek = x.dow)
							and (A_SmsTaskManager_RunPeriods.StartTime = x.start) 
							and (A_SmsTaskManager_RunPeriods.FinishTime = x.fin) )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_SmsTaskManager_RunPeriods (TaskId, DayOfWeek, StartTime, FinishTime)
		Select Distinct @taskid, x.dow, x.start, x.fin
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_SmsTaskManager_RunPeriods w 
							Where (w.TaskId = @taskid) 
								and (w.DayOfWeek = x.dow) 
								and (w.StartTime = x.start) 
								and (w.FinishTime = x.fin) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SvcTaskManager_Task_ClearPeriods' ))
	Drop procedure dbo.A_SvcTaskManager_Task_ClearPeriods
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_SvcTaskManager_Task_PeriodsSave' ))
	Drop procedure dbo.A_SvcTaskManager_Task_PeriodsSave
GO

Create procedure dbo.A_SvcTaskManager_Task_PeriodsSave
	@taskid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( dow int, start datetime, fin datetime )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select dow, start, fin
		From OpenXml (@idoc, '/items/p', 2)	
		With ( 
			dow int '@dow',
			start datetime '@start',
			fin datetime '@fin'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_SvcTaskManager_RunPeriods
	Where (TaskId = @taskid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where (A_SvcTaskManager_RunPeriods.DayOfWeek = x.dow)
							and (A_SvcTaskManager_RunPeriods.StartTime = x.start) 
							and (A_SvcTaskManager_RunPeriods.FinishTime = x.fin) )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_SvcTaskManager_RunPeriods (TaskId, DayOfWeek, StartTime, FinishTime)
		Select Distinct @taskid, x.dow, x.start, x.fin
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_SvcTaskManager_RunPeriods w 
							Where (w.TaskId = @taskid) 
								and (w.DayOfWeek = x.dow) 
								and (w.StartTime = x.start) 
								and (w.FinishTime = x.fin) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Clients_Contact_Delete' ))
	Drop procedure dbo.A_Clients_Contact_Delete
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Clients_Contact_Save' ))
	Drop procedure dbo.A_Clients_Contact_Save
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Clients_Contacts_Save' ))
	Drop procedure dbo.A_Clients_Contacts_Save
GO

Create procedure dbo.A_Clients_Contacts_Save
	@clientid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml 
	( 
		id uniqueidentifier Primary key, 
		typeid uniqueidentifier, 
		content nvarchar(500) collate database_default, 
		isdefault bit 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, typeid, content, isdefault
		From OpenXml (@idoc, '/items/c', 2)	
		With ( 
			id varchar(50) '@id',
			typeid varchar(50) '@typeid',
			content nvarchar(500) '@cont',
			isdefault bit '@isdef'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ClientContacts
	Where (IdClient = @clientid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where (A_ClientContacts.Id = x.id) )
							
    -- Обновляем тех, у которых что-то изменилось
	Update w
	Set w.IdContact = x.typeid,
		w.Content = x.content,
		w.IsDefaultPhone = x.isdefault
	From dbo.A_ClientContacts w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IdClient = @clientid)
		and ((w.IdContact != x.typeid)
			or (w.Content != x.content)
			or (w.IsDefaultPhone != x.isdefault) )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ClientContacts ([Id], IdClient, IdContact, Content, IsDefaultPhone)
		Select Distinct x.id, @clientid, x.typeid, x.content, x.isdefault
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_ClientContacts w 
							Where (w.IdClient = @clientid) 
								and (w.Id = x.id) )
	
end
GO


----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Clients_ClientGroup_ClearForClient' ))
	Drop procedure dbo.A_Clients_ClientGroup_ClearForClient
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Clients_ClientGroups_Save' ))
	Drop procedure dbo.A_Clients_ClientGroups_Save
GO

Create procedure dbo.A_Clients_ClientGroups_Save
	@clientid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( id uniqueidentifier Primary key )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id
		From OpenXml (@idoc, '/items/cg', 2)	
		With ( id varchar(50) '@id' )	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_Clients_Groups
	Where (IdClient = @clientid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where (A_Clients_Groups.IdGroup = x.id) )
						
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Clients_Groups ([Id], IdClient, IdGroup)
		Select Distinct NewId(), @clientid, x.id
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_Clients_Groups w 
							Where (w.IdClient = @clientid) 
								and (w.IdGroup = x.id) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Redirect_TimeTable_DeleteById' ))
	Drop procedure dbo.A_Redirect_TimeTable_DeleteById
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Redirect_TimeTable_PeriodsSave' ))
	Drop procedure dbo.A_Redirect_TimeTable_PeriodsSave
GO

Create procedure dbo.A_Redirect_TimeTable_PeriodsSave
	@redirid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml ( dow int, start datetime, fin datetime )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select dow, start, fin
		From OpenXml (@idoc, '/items/p', 2)	
		With ( 
			dow int '@dow',
			start datetime '@start',
			fin datetime '@fin'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_Redirect_TimeTable
	Where (IdRedir = @redirid)
		and not exists ( Select 1 
						From #tmp_xml x 
						Where (A_Redirect_TimeTable.DayOfWeek = x.dow)
							and (A_Redirect_TimeTable.StartTime = x.start) 
							and (A_Redirect_TimeTable.StopTime = x.fin) )
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Redirect_TimeTable (IdRedir, DayOfWeek, StartTime, StopTime)
		Select Distinct @redirid, x.dow, x.start, x.fin
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_Redirect_TimeTable w 
							Where (w.IdRedir = @redirid) 
								and (w.DayOfWeek = x.dow) 
								and (w.StartTime = x.start) 
								and (w.StopTime = x.fin) )
	
end
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Redirect_Abonents_DeleteById' ))
	Drop procedure dbo.A_Redirect_Abonents_DeleteById
GO

----------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Redirect_AbonentsSave' ))
	Drop procedure dbo.A_Redirect_AbonentsSave
GO

Create procedure dbo.A_Redirect_AbonentsSave
	@redirid uniqueidentifier,
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml			
	Create table #tmp_xml 
	( 
		id uniqueidentifier, 
		num nvarchar(50) collate database_default, 
		descr nvarchar(255) collate database_default, 
		type int 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, num, descr, type
		From OpenXml (@idoc, '/items/ab', 2)	
		With ( 
			id varchar(50) '@id',
			num nvarchar(50) '@num',
			descr nvarchar(255) '@descr',
			type int '@type'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_Redirect_Abonents
	Where (IdRedir = @redirid)
		and not exists( Select 1 
						From #tmp_xml x 
						Where (x.type = A_Redirect_Abonents.Type)
							and (((x.id != '00000000-0000-0000-0000-000000000000') and (x.id = A_Redirect_Abonents.IdAbonent )) 
								or ((not x.num is NULL) and (x.num = A_Redirect_Abonents.Number ))))
		
	-- Обновляем тех, у которых что-то изменилось
	Update w
	Set w.Description = x.descr		
	From dbo.A_Redirect_Abonents w, #tmp_xml x
	Where (w.IdRedir = @redirid)
		and (x.type = w.Type)
		and (((x.id != '00000000-0000-0000-0000-000000000000') and (x.id = w.IdAbonent))
			or ((not x.num is NULL) and (x.num = w.Number)))
		and ((w.Description != x.descr))
	
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Redirect_Abonents (IdRedir, IdAbonent, Number, Description, Type)
		Select Distinct @redirid, x.id, x.num, x.descr, x.type
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_Redirect_Abonents w 
							Where (w.IdRedir = @redirid)
								and (x.type = w.Type)
								and (((x.id != '00000000-0000-0000-0000-000000000000') and (x.id = w.IdAbonent )) 
									or ((not x.num is NULL) and (x.num = w.Number ))))								
	
end
GO
