﻿USE [oktell]
GO

IF OBJECT_ID('dbo.H_Authorizeduserid_SPID') IS NOT NULL
    Drop Table dbo.H_Authorizeduserid_SPID
GO

Create Table dbo.H_Authorizeduserid_SPID 
(
	[authorizeduser_name] nvarchar(500) collate database_default, 
	[getdate] DATETIME default getdate(), 
	[tr_id] varchar(255) collate database_default, 
	[sql_text] nvarchar(4000) collate database_default
)

Create Index  H_Authorizeduserid_SPID_tr_id_indx ON dbo.H_Authorizeduserid_SPID ([tr_id] DESC)

Create Index  H_Authorizeduserid_SPID_getdate_indx ON dbo.H_Authorizeduserid_SPID ([getdate] DESC)
GO

IF OBJECT_ID('dbo.H_HISTORY') IS NOT NULL
    Drop table dbo.H_HISTORY
GO

Create table [dbo].[H_HISTORY] (
    [numid] bigint identity(1,1) not null Primary Key,
    [id] uniqueidentifier,
    [type] varchar(10) collate database_default NULL,
    [table_name] nvarchar(100) collate database_default NULL,
    [authorizeduser_name] nvarchar(500) collate database_default NULL,
    [columns_name] nvarchar(4000) collate database_default NULL,
    [old_values] nvarchar(4000) collate database_default NULL,
    [new_values] nvarchar(4000) collate database_default NULL,
    [side] smallint NULL,
    [getdate] datetime NULL DEFAULT (getdate()),
    [full_query_transaction] nvarchar(4000) collate database_default NULL,
    [full_query_transactionT] nvarchar(4000) collate database_default NULL,
    [tr_id] varchar(255) collate database_default,
    [col_id] varchar(500) collate database_default
)

Create Index H_HISTORY_type_getdate_indx on H_HISTORY ([type], [getdate] DESC)
GO 


IF OBJECT_ID('dbo.H_Authorizeduserid_SPID_Save') IS NOT NULL
    Drop procedure dbo.H_Authorizeduserid_SPID_Save
GO
   
Create procedure dbo.H_Authorizeduserid_SPID_Save 
    @authorizeduserid uniqueidentifier = NULL
as
begin

    If (@authorizeduserid IS NOT NULL) begin
    
        Declare @authorizeduser_name nvarchar(500),
			@tr_id varchar(255), 
			@sql_text nvarchar(4000), 
			@sql_handle varbinary(255)
			
        Set @authorizeduser_name = Coalesce(@authorizeduserid, SUSER_SNAME())
        
        
        If ((Select Cast(Replace(Left(Cast(SERVERPROPERTY('productversion') as varchar),2),'.','') as INT)  ) > 8) begin
        
			-- MSSQL 2005>
            Set @tr_id = ( Select Cast(transaction_id as varchar(255)) 
							From sys.dm_tran_current_transaction )
							
            Select top 1 @sql_handle = r.sql_handle
            From sys.dm_exec_requests r 
            Where r.transaction_id = @tr_id
                    
            Select top 1 @sql_text = s_est. [text] 
            From sys.dm_exec_sql_text (@sql_handle) s_est
            
        end else begin
        
			-- MSSQL 2000
            Declare @stmt_start int,
                @stmt_end int

            Execute sp_getbindtoken @tr_id OUTPUT
            
            Select top 1
                @sql_handle = sql_handle,
               @stmt_start = case stmt_start when 0 then 0 else stmt_start / 2 end,
               @stmt_end = case stmt_end when -1 then -1 else stmt_end / 2 end
            From Master.dbo.sysprocesses
            Where spid = @@SPID
            Order By ecid

            Select @sql_text = 
                Substring ( text,
                        Coalesce( NullIf(@stmt_start, 0), 1),
                        Case @stmt_end
                            When -1
                                Then DataLength(text)
                            Else
                                (@stmt_end - @stmt_start)
                            End
                    )
            From ::fn_get_sql(@sql_handle)
            
            Create table #temp (
				[EventType] nvarchar (30) collate database_default, 
				[Parameters] int, 
				[EventInfo] nvarchar(4000) collate database_default
			)
			
            Insert into #temp
				Exec sp_executesql N'DBCC inputbuffer (@@spid) WITH NO_INFOMSGS'
				
            Set @sql_text = Coalesce( NullIf(@sql_text,''),(Select top 1 EventInfo From #temp))
        End
        
        Insert Into [dbo].H_Authorizeduserid_SPID ([authorizeduser_name], [getdate], [tr_id], [sql_text]) 
        Values (@authorizeduser_name, getdate(), @tr_id, @sql_text)
        
    End
End
GO