﻿---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerStreams_Save'))
	Drop procedure dbo.A_Admin_OfficeMap_ServerStreams_Save
GO

Create procedure [dbo].[A_Admin_OfficeMap_ServerStreams_Save]
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		name nvarchar(500) collate database_default, 
		type int, 
		srvid uniqueidentifier, 
		sipuser nvarchar(500) collate database_default, 
		regtype int, 
		gateid uniqueidentifier, 
		dialmode int,
		sippass nvarchar(500) collate database_default,
		sipdisplay nvarchar(500) collate database_default,
		defcallerid nvarchar(50) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, name, type, srvid, sipuser, regtype, gateid, dialmode, sippass, sipdisplay, defcallerid
		From OpenXml (@idoc, '/items/stream', 2)	
		With ( 
			id varchar(50) '@id',
			name nvarchar(500) '@name',
			type int '@type',
			srvid varchar(50) '@srvid',
			sipuser nvarchar(500) '@sipuser',
			regtype int '@regtype',
			gateid varchar(50) '@gate',
			dialmode int '@dialmode',
			sippass nvarchar(500) '@sippass',
			sipdisplay nvarchar(500) '@sipdisp',
			defcallerid nvarchar(50) '@dcid'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerStreams
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerStreams.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IdServer = x.srvid,
		w.Type = x.type,
		w.Account = x.sipuser,
		w.RegType = x.regtype,
		w.GateId = x.gateid,
		w.DialMode = x.dialmode,
		w.SipPass = x.sippass,
		w.SipDisplay = x.sipdisplay,
		w.DefCallerId = x.defcallerid
	From dbo.A_ServerStreams w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Name != x.name)
			or (w.IdServer != x.srvid)
			or (w.Type != x.type)
			or (w.Account != x.sipuser)
			or (w.RegType != x.regtype)
			or (w.GateId != x.gateid)
			or (w.DialMode != x.dialmode)
			or (w.SipPass != x.sippass)
			or (w.SipDisplay != x.sipdisplay)
			or (w.DefCallerId != x.defcallerid))			
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerStreams (Id, Name, Type, IdServer, Account, RegType, GateId, DialMode, SipPass, SipDisplay, DefCallerId)
		Select Distinct id, name, type, srvid, sipuser, regtype, gateid, dialmode, sippass, sipdisplay, defcallerid
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerStreams w Where w.Id = x.id )
		
end
GO


----------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_HolidaysUser_Save'))
	Drop procedure dbo.A_HolidaysUser_Save
GO

Create procedure [dbo].A_HolidaysUser_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
		
	Select * FROM A_HolidaysUser
	
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		idx int identity (1,1) primary key,
		date datetime,
		type bit, 
		asdayofweek int, 
		description nvarchar(1000) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select date, type, asdayofweek, description
		From OpenXml (@idoc, '/items/h', 2)	
		With ( 
			date datetime '@date',
			type bit '@type',
			asdayofweek int '@asdow',
			description nvarchar(1000) '@descr'
		)	
		
	If object_id('tempdb..#tmp_xml2') is not NULL
		Drop table #tmp_xml2
	Create table #tmp_xml2 
	( 	
		date datetime primary key,
		type bit, 
		asdayofweek int, 
		description nvarchar(1000) collate database_default
	)
	
	-- Уникальные строчки вырезаем
	Insert into #tmp_xml2 ( date, type, asdayofweek, description )
		Select x.date, x.type, x.asdayofweek, x.description
		From #tmp_xml x
			Inner join( Select x.date, Min(idx) as idx
						From #tmp_xml x
						Group by x.date )a 
					on a.idx = x.idx
			
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_HolidaysUser
	Where not exists ( Select 1 
						From #tmp_xml2 x 
						Where (A_HolidaysUser.Date = x.date))
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.AsDayOfWeek = x.asdayofweek,
		w.Description = x.description
	From dbo.A_HolidaysUser w, #tmp_xml2 x
	Where (w.Date = x.date)
		and ((w.Type != x.type)
			or (w.AsDayOfWeek != x.asdayofweek)
			or (w.Description != x.description))			
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_HolidaysUser (Date, Type, AsDayOfWeek, Description)
		Select Distinct date, type, asdayofweek, description
		From #tmp_xml2 x
		Where not exists ( Select 1 
							From dbo.A_HolidaysUser w 
							Where (w.Date = x.date) )
		
end
GO


-------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_UserResultsProject_SaveMulti'))
	Drop procedure dbo.A_TaskManager_UserResultsProject_SaveMulti
GO

Create procedure [dbo].A_TaskManager_UserResultsProject_SaveMulti
	@idproject uniqueidentifier,	
	@xml ntext,
	@authorizeduserid uniqueidentifier
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid


	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id int,
		name nvarchar(200) collate database_default,
		isoutput bit,
		success bit,
		isdefault bit
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/ur', 2)	
		With ( 
			id int '@id',
			name nvarchar(200) '@nm',
			isoutput bit '@out',
			success bit '@scs',
			isdefault bit '@def'
		)	
		
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_TaskManager_CardUserResultsProject
	Where (ProjectId = @idproject)
		and not exists( Select 1 
						From #tmp_xml x 
						Where (dbo.A_TaskManager_CardUserResultsProject.ProjectId = @idproject)
							and (dbo.A_TaskManager_CardUserResultsProject.Id = x.id))
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IsOutput = x.isoutput,
		w.Success = x.success,
		w.IsDefault = x.isdefault
	From dbo.A_TaskManager_CardUserResultsProject w, #tmp_xml x
	Where (w.ProjectId = @idproject)
		and (w.Id = x.id)
		and ((w.Name != x.name)
			or (w.IsOutput != x.isoutput)
			or (w.Success != x.success)
			or (w.IsDefault != x.isdefault))						
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_TaskManager_CardUserResultsProject (ProjectId, Id, Name, IsOutput, Success, IsDefault)
		Select Distinct @idproject, x.id, x.name, x.isoutput, x.success, x.isdefault
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_TaskManager_CardUserResultsProject w 
							Where (w.ProjectId = @idproject)
								and (w.Id = x.id))
									
end
GO