﻿if not ( object_id ( 'dbo.A_CallCenter_Temp_UpdateOperatorStates' ) is NULL )
	Drop procedure dbo.A_CallCenter_Temp_UpdateOperatorStates
GO

Create Procedure [dbo].[A_CallCenter_Temp_UpdateOperatorStates]
	@dtnow datetime = null
as 
begin

	set dateformat dmy

	if ( @dtnow is null )	
		set @dtnow = DateAdd ( ss, -30, GetDate() )

	if not exists ( Select * FROM master..sysdatabases where name like 'oktell_cc_temp' )
		return

	Declare @sqltext nvarchar ( 4000 )

	Declare @idempty uniqueidentifier
	Set @idempty = '00000000-0000-0000-0000-000000000000'
	
	-- Всех операторов в таблицу
	Declare @settable table ( Id uniqueidentifier, Name nvarchar ( 500 ) collate database_default)
	Insert into @settable
		Select Id, Name From A_Users u 
			Inner join A_UserParams up on u.Id = up.IdUser
		Where IsOperator = 1

	-- N минут назад
	Declare @dt10m DateTime
	Set @dt10m = DateAdd ( mi, -20, @dtnow )

	-- формируем общую выборку из статистики коммутаций
	Declare @dtsall DateTime
	Select @dtsall = Min ( LastDT ) 
	From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi
		Inner join @settable up on oi.Id = up.Id

	-- Если нет распоследнего - расчитываем за 10 дней.
	if ( @dtsall is null )
		Set @dtsall = DateAdd ( d, -10, @dtnow )

	-- Новое (06.2010)
	-- Если распоследний менее чем 10 минут до текущего момента - считать будем 10 минут.
	if ( @dtsall > @dt10m )
		Set @dtsall = @dt10m
	
	---- Кэш для A_Stat_Connections_1x1
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#statconnections' ) )
		Drop Table #statconnections	
	Create table #statconnections 
	( 
		FromCC bit, 
		IsBetweenUsers bit, 
		TimeStart DateTime, 
		TimeAnswer DateTime, 
		TimeStop DateTime, 
		AUserId uniqueidentifier, 
		BUserId uniqueidentifier 
	)

	Set @sqltext = 'Select FromCC, IsBetweenUsers, TimeStart, TimeAnswer, TimeStop, AUserId, BUserId 
		From A_Stat_Connections_1x1 sc 
		Where ( TimeStart > @dtt1 Or TimeStop > @dtt1 Or TimeStop is null ) And ( TimeStart < @dtnow )'
	
	Insert Into #statconnections
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	---- Кэш для A_CallCenter_UserStateHistory
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#ccuserstatehistory' ) )
		Drop Table #ccuserstatehistory
	Create table #ccuserstatehistory 
	(
		OperatorId uniqueidentifier, 
		State int, 
		TimeEnter DateTime, 
		TimeLeave DateTime, 
		Info nvarchar ( 100 ) collate database_default, 
		ICode int 
	)

	Set @sqltext = 'Select OperatorId, State, TimeEnter, TimeLeave, case when selfinfo = 1 then Info else '''' end Info, ICode
		From A_CallCenter_UserStateHistory 
		Where ( TimeEnter > @dtt1 Or TimeLeave > @dtt1 Or TimeLeave is null ) And ( TimeEnter < @dtnow )'

	Insert Into #ccuserstatehistory
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	
	---- Кэш для A_TaskManager_UserInTask
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#userintask' ) )
		Drop Table #userintask
	Create table #userintask 
	( 
		Id uniqueidentifier, 
		UserId uniqueidentifier, 
		TaskId uniqueidentifier, 
		EffortId uniqueidentifier, 
		TimeStart DateTime, 
		TimeStopPrior DateTime, 
		TimeStopWaitCall DateTime, 
		TimeStopBackCall DateTime, 
		TimeStopDialog DateTime, 
		TimeStopPost DateTime 
	)

	Set @sqltext = 'Select Id, UserId, TaskId, Effortid, IsNull ( TimeStart, @dtnow ), IsNull ( TimeStopPrior, @dtnow ), IsNull ( TimeStopWaitCall, @dtnow ), IsNull ( TimeStopBackCall, @dtnow ), IsNull ( TimeStopDialog, @dtnow ), IsNull ( TimeStopPost, @dtnow )
		From A_TaskManager_UserInTask 
		Where ( TimeStart >= @dtt1 or TimeStopPost >= @dtt1 or TimeStopPost is null ) And ( TimeStart < @dtnow )'

	Insert Into #userintask
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	
	---- Кэш для части предыдущих данных из oktell_cc_temp.dbo.A_Cube_CC_OperatorStates
	Declare @maxid table 
	( 
		IdOperator uniqueidentifier, 
		MaxId bigint 
	)
		
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#tmpos04' ) )
		Drop Table #tmpos04
	Create table #tmpos04 ( IdOperator uniqueidentifier, MaxId bigint )

	-- Новое (06.2010)
	--   микс со временем, поскольку удалять записи будем все равно..
	Set @sqltext = 'Select IdOperator,  Max ( Id )
		From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates]
		Where DateTimeStart < @dtm
		Group By IdOperator'

	Insert Into #tmpos04
		exec sp_executesql @sqltext, N'@dtm datetime', @dt10m
	Insert into @maxid
		Select * FROM #tmpos04	

	drop table #tmpos04
		

	-- Распоследнее время для каждого оператора может быть уникальным
	Declare @umaxdts table ( UserId uniqueidentifier, MaxDTS DateTime  )

	Insert into @umaxdts
		Select IdOperator, Max ( DateTimeStop ) 
		From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] With ( NOLOCK )
		Group By IdOperator

	-- Новое (06.2010)
	-- Будем перерасчитывать не менее 10 минут по любому...
	Update @umaxdts
	Set MaxDTS = @dt10m
	Where MaxDTS > @dt10m


	-- цикл по операторам
	Begin Tran TranUpdateOperatorStates

		Declare ucursor Cursor LOCAL FAST_FORWARD For 
			Select Id, Name 
			From @settable
			
			
		Declare @idoperator uniqueidentifier, @nameoperator nvarchar ( 100 )	


		Open ucursor
		Fetch Next 
			From ucursor 
			Into @idoperator, @nameoperator

		While ( @@fetch_status = 0 ) begin

			Declare @id uniqueidentifier
			Select @id = Id from [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
			Where Id = @idoperator
			
			if ( @@rowcount = 0 )
				Insert 
				Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] ( Id, Name ) 
				Values ( @idoperator, @nameoperator )
						
			-- определяем время последнего изменения состояния
			Declare @dts DateTime, @dts1 DateTime
			
			Select @dts = MaxDTS
			From @umaxdts
			Where UserId = @idoperator
			
			Select @dts1 = LastDT
			From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] 
			Where Id = @idoperator
			
			if ( @dts1 > @dts ) or ( @dts is null )
				Set @dts = @dts1

			if ( @dts is null )
				set @dts = '01.01.2000'

			if ( @dts > @dt10m )
				set @dts = @dt10m


			-- обновление производим для операторов не чаще одного раза в минуту
			if ( Cast ( @dtnow - @dts as float ) * 1440 > 1 ) begin		
				
				-- Итоговые непересекающиеся состояния
				/*
				PriorHandle		= 1,
				WaitingCall		= 2,
				BackRing		= 3,
				TalkAbonent		= 4,
				TalkInside		= 5,
				InsideTalk		= 6,
				PostHandle		= 7,
				TalkOther		= 8,
				CCLunch			= 9,
				CCManual		= 12,
				CCReady			= 10,
				System			= 11
				*/
				
				-- Приоритеты (чем меньше, тем выше)
				--   то есть если нет состояния с меньшим приоритетом, подставляется следующее из имеющихся с понижением приоритета
				/*
				UserInTask		= 1,
				Lunch			= 2,
				Commutations	= 3,
				ManualCC		= 4,
				OnlineCC		= 4,
				System			= 5,
				*/

				-----------------------------------------------------
				-- 1. вторая таблица (статусы оператора в КЦ) ----------
				-----------------------------------------------------
				Declare @tempo2 table
				(
					State int not null,
					Priority int,
					DateTimeStart DateTime,
					DateTimeStop DateTime,
					Id int Identity ( 1, 1 ) not null primary key,
					Info nvarchar ( 100 ) collate database_default,
					ICode int
				)

				Delete From @tempo2

				---------------------
				-- 0-в системе, <= 1-в коллцентре, <= ( 2-онлайн, 3-перерыв, 4-ручной режим )
				---------------------
				Insert into @tempo2 ( State, Priority, DateTimeStart, DateTimeStop, Info, ICode )
					Select State, 
						   Case when (State = 3) --Перерыв
									then 2 
								when (State = 4) --Ручной режим
									then 4 
								else 4 --Онлайн состояния
							 end as Priority, 
						   TimeEnter, IsNull ( TimeLeave, @dtnow ) as DateTimeStop, Info, ICode
					From #ccuserstatehistory
					Where OperatorId = @idoperator
						And State in ( 2, 3, 4 )
						And ( TimeEnter > @dts Or TimeLeave > @dts Or TimeLeave is null )
				
				Update @tempo2
				Set State = Case when (State = 3)--Перерыв
									then 9 
								 when (State = 4)--
									then 12
								 else 10--Прочие онлайны (они менее приоритетны)
						      end

				
				-- если есть активное состояние кц - обновляем
				if ( @@rowcount > 0 ) begin	
				
					-- докидываем информацию по присутствию в системе
					Insert into @tempo2 ( State, Priority, DateTimeStart, DateTimeStop, Info, ICode )
						Select 11 as State, 
							   5 as Priority,
							   TimeEnter, IsNull ( TimeLeave, @dtnow ) as DateTimeStop, Info, ICode
						From #ccuserstatehistory
						Where OperatorId = @idoperator
							And State = 0
							And ( TimeEnter > @dts Or TimeLeave > @dts Or TimeLeave is null )

					Update @tempo2			
					Set DateTimeStart = Case When (DateTimeStart < @dts) then @dts else DateTimeStart end,
						DateTimeStop = Case When (DateTimeStop > @dtnow) then @dtnow else DateTimeStop end
					
			
					-----------------------------------------------------
					-- 2. Первая таблица (статусы оператора в задачах) -----
					-----------------------------------------------------
					Declare @tempo1 table
					(
						IdTask uniqueidentifier,
						IsOutput bit,
						IdOperator uniqueidentifier,
						CallResult int,
						State int not null,
						StateCat int,
						DateTimeStart DateTime,
						DateStart DateTime,
						TimeStart DateTime,
						DateTimeStop DateTime,
						LenTime Float,
						IdEffort uniqueidentifier,
						Priority int,
						Id int identity ( 1, 1 ) not null primary key
					)

					Delete From @tempo1
					
					Declare @uito table 
					(
						Id uniqueidentifier,
						UserId uniqueidentifier,
						TaskId uniqueidentifier,	
						EffortId uniqueidentifier,
						TimeStart DateTime,
						TimeStopPrior DateTime,
						TimeStopWaitCall DateTime,
						TimeStopBackCall DateTime,
						TimeStopDialog DateTime,
						TimeStopPost DateTime
					)

					Delete From @uito

					Insert Into @uito
						Select *
						From #userintask
						Where UserId = @idoperator
				
				
					-- Предвызывная обработка (состояние 1, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat, 
							DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 1, 0, 
							TimeStart, TimeStopPrior
						From @uito
						Where Cast ( TimeStopPrior - TimeStart as float ) * 86400 > 0
					
					
					-- Ожидание дозвона (состояние 2, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 2, 0, 
							TimeStopPrior, TimeStopWaitCall
						From @uito
						Where Cast ( TimeStopWaitCall - TimeStopPrior as float ) * 86400 > 0
					
					
					-- Обратный вызов (состояние 3, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 3, 0, 
							TimeStopWaitCall, TimeStopBackCall
						From @uito
						Where Cast ( TimeStopBackCall - TimeStopWaitCall as float ) * 86400 > 0

					
					-- Разговор (состояние 6, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 6, 0, 
							TimeStopBackCall, TimeStopDialog
						From @uito
						Where Cast ( TimeStopDialog - TimeStopBackCall as float ) * 86400 > 0

					
					-- Поствызывная обработка (состояние 7, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 7, 0, 
							TimeStopDialog, TimeStopPost
						From @uito
						Where Cast ( IsNull ( TimeStopPost, @dtnow ) - IsNull ( TimeStopDialog, @dtnow ) as float ) * 86400 > 0



					-- время начала и время конца по периоду
					Update @tempo1 
					Set DateTimeStart = case when ( DateTimeStart < @dts ) then @dts 
											else DateTimeStart end,
		 				DateTimeStop = case when ( DateTimeStop > @dtnow ) then @dtnow 
											when ( DateTimeStop < @dts ) then @dts 
											else DateTimeStop end

					Delete From @tempo1
					Where DateTimeStart = DateTimeStop


					-- проставляем флаг исх/вх задачи
					Update tmpo
					Set IsOutput = t.IsOutputTask
					From @tempo1 tmpo, 
						A_TaskManager_Tasks t
					Where tmpo.IdTask = t.Id

					-- проставляем результат выполнения задачи
					Update tmpo
					Set CallResult = tic.CallResult
					From @tempo1 tmpo, 
						A_TaskManager_Task_IncomingCalls tic
					Where tmpo.IdEffort = tic.Id

					Update tmpo
					Set CallResult = ste.EResult
					From @tempo1 tmpo, 
						A_TaskManager_SubTaskEfforts ste
					Where tmpo.IdEffort = ste.Id

			
					-----------------------------------------------------
					-- 3. третья таблица (Разговоры) -----------------------
					-----------------------------------------------------
					
					Declare @tempo3 table
					(
						State int not null,
						Priority int,
						DateTimeStart DateTime,
						DateTimeStop DateTime,
						Id int identity ( 1, 1 ) not null primary key
					)
					
					Delete From @tempo3
								
					Insert into @tempo3
						Select 8, 0, TimeStart, isnull ( TimeStop, @dtnow )
						From #statconnections
						Where ( AUserId = @idoperator Or BUserId = @idoperator )
							And ( TimeStart > @dts Or TimeStop > @dts Or TimeStop is null )
							And ( FromCC = 0 )
						

					Update @tempo3
					Set DateTimeStart = Case When DateTimeStart < @dts then @dts else DateTimeStart end,
						Priority = Case When State = 8 then 3 else 0 end
										

					-----------------------------------------------------
					-- Формируем времянку по точкам ---------------------
					-----------------------------------------------------

					-- Select * FROM @tempo1 Order by DateTimeStart
					-- Select * FROM @tempo2 Order by DateTimeStart
					-- Select * From @tempo3 Order by DateTimeStart
				
					Declare @tempo table
					(
						TableN int,
						TableID int,
						State int not null,
						Priority int,
						DT DateTime,
						NK bit
					)

					Delete from @tempo

					Insert into @tempo
						Select 1, Id, State, Priority, DateTimeStart, 0
						From @tempo1

					Insert into @tempo
						Select 1, Id, State, Priority, DateTimeStop, 1
						From @tempo1

					Insert into @tempo
						Select 2, Id, State, Priority, DateTimeStart, 0
						From @tempo2

					Insert into @tempo
						Select 2, Id, State, Priority, DateTimeStop, 1
						From @tempo2

					Insert into @tempo
						Select 3, Id, State, Priority, DateTimeStart, 0
						From @tempo3

					Insert into @tempo
						Select 3, Id, State, Priority, DateTimeStop, 1
						From @tempo3
					

					-- В Tempo2 лежат ICode, но так как состояние перерыва фактически будет по концу коммутации, если оная была
					--   а пересечение с Tempo2 по времени, то нужно добавить в Tempo2 куски из других таблиц, но с их временами,
					--   и расширить предыдущим ICode для пересечения


					-----------------------------------------------------
					-- пробегаем курсором -------------------------------
					-----------------------------------------------------

					Declare @res table ( TableN int, Id int, State int not null, Priority int, DStart DateTime, DStop DateTime )
					Delete From @res

					-- определяем статусы и приоритеты на начало периода расчета
					-- приоритеты: 	0 - коммутации внутри задачи ( попытки )
					--		1 - состояния оператора внутри задачи ( попытки )
					--		2 - перерывы
					--		3 - прочие коммутации
					--		4 - онлайн в КЦ, ручной режим в КЦ
					--		5 - онлайн в системе
					Declare @p0 int, @p1 int, @p2 int, @p3 int, @p4 int, @p5 int
					Declare @s0 int, @s1 int, @s2 int, @s3 int, @s4 int, @s5 int
					Declare @t0 int, @t1 int, @t2 int, @t3 int, @t4 int, @t5 int
					Declare @i0 int, @i1 int, @i2 int, @i3 int, @i4 int, @i5 int

					-- Новое (06.2010)
					--   Не применяем предыдущую историю количеств. Всегда должно в ноль сводиться
					Select @p0 = 0, @p1 = 0, @p2 = 0, @p3 = 0, @p4 = 0, @p5 = 0
					Select @s0 = IsNull ( S0, 0 ), @s1 = IsNull ( S1, 0 ), @s2 = IsNull ( S2, 0 ), @s3 = IsNull ( S3, 0 ), @s4 = IsNull ( S4, 0 ), @s5 = IsNull ( S5, 0 )
					From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] 
					Where Id = @idoperator
				
					-- формируем сдвоенную таблицу	
					Declare @tempo_ordered table
					(
						Id int identity ( 1, 1 ) not null,
						TableN int,
						TableID int,
						State int not null,
						Priority int,
						DT DateTime,
						NK bit
					)

					Delete From @tempo_ordered

					Insert into @tempo_ordered ( TableN, TableID, State, Priority, DT, NK )
						Select TableN, TableID, State, Priority, DT, NK From @tempo
						Order By DT

					
					Declare @tempo_joined table
					(
						o1 int, 
						o2 int, 
						s1 int not null, 
						p1 int not null, 
						s2 int not null, 
						p2 int not null, 
						dstart datetime, 
						dstop datetime, 
						nk1 bit, 
						nk2 bit, 
						t1 int, 
						t2 int, 
						i1 int, 
						i2 int
					)
					Delete From @tempo_joined
				
					Insert into @tempo_joined 
						Select t1.Id, t2.Id, t1.State as s1, t1.Priority as p1, t2.State as s2, t2.Priority as p2,
							t1.DT as dstart, t2.DT as dstop, t1.NK as nk1, t2.NK as nk2,
							t1.TableN, t2.TableN, t1.TableID, t2.TableID
						From @tempo_ordered t1
							Inner join @tempo_ordered t2 on t1.Id + 1 = t2.Id


					-- курсором бежим и заполняем таблицу результата 
					Declare tcursor Cursor LOCAL FAST_FORWARD For 
						Select o1, o2, s1, p1, s2, p2, dstart, dstop, nk1, nk2, t1, t2, i1, i2 
						From @tempo_joined
						Order by o1						

					Declare @o1 int, @o2 int
					Declare @st1 int, @pr1 int, @st2 int, @pr2 int, @dstart DateTime, @dstop DateTime, @nk1 bit, @nk2 bit, @tb1 int, @tb2 int, @id1 int, @id2 int
					Declare @actualst int, @actualpr int, @actualt int, @actualid int
					Declare @lastst int, @lastpr int, @lastt int, @lastid int, @lastdstart datetime, @lastdstop datetime
					Declare @dontinsert bit, @updatelast bit
					Declare @insertindex int

					set @insertindex = 0
					set @lastt = 0
					set @dontinsert = 1 -- чтобы при втором обновлении (при существующей таблице данных) не вставляло имеющееся
				
					Open tcursor
					Fetch Next 
						From tcursor 
						Into @o1, @o2, @st1, @pr1, @st2, @pr2, @dstart, @dstop, @nk1, @nk2, @tb1, @tb2, @id1, @id2

					While (@@FETCH_STATUS = 0) Begin
		
						--- оцениваем перед началом -----		
						if ( @nk1 = 0 ) begin			
						
							if ( @pr1 = 0 ) begin
			    			
			    				set @p0 = @p0 + 1
				    			set @s0 = @st1				
								set @t0 = @tb1	
								set @i0 = @id1
							
							end else if ( @pr1 = 1 ) begin
							
								set @p1 = @p1 + 1
				    			set @s1 = @st1
								set @t1 = @tb1
								set @i1 = @id1
								
							end else if ( @pr1 = 2 ) begin
							
			    				set @p2 = @p2 + 1
				    			set @s2 = @st1
								set @t2 = @tb1
								set @i2 = @id1
								
							end else if ( @pr1 = 3 ) begin
				    				
				    			set @p3 = @p3 + 1
				    			set @s3 = @st1
								set @t3 = @tb1
								set @i3 = @id1
								
							end else if ( @pr1 = 4 ) begin
			    					
			    				set @p4 = @p4 + 1
				    			set @s4 = @st1
								set @t4 = @tb1
								set @i4 = @id1
								
							end else if ( @pr1 = 5 ) begin
								
								set @p5 = @p5 + 1
								set @s5 = @st1
								set @t5 = @tb1
								set @i5 = @id1
							
							end
						end


						--- производим расчет и модернизацию
						set @dontinsert = 0
						if ( @p0 > 0 ) begin
						
							set @actualst = @s0
							set @actualpr = 0
							set @actualt = @t0
							set @actualid = @i0
						
						end else if ( @p1 > 0 ) begin
						
							set @actualst = @s1
							set @actualpr = 1
							set @actualt = @t1
							set @actualid = @i1

						end else if ( @p2 > 0 )	begin
						
							set @actualst = @s2
							set @actualpr = 2
							set @actualt = @t2
							set @actualid = @i2
						
						end else if ( @p3 > 0 ) begin
						
							set @actualst = @s3
							set @actualpr = 3
							set @actualt = @t3
							set @actualid = @i3
						
						end else if ( @p4 > 0 )begin
						
							set @actualst = @s4
							set @actualpr = 4
							set @actualt = @t4
							set @actualid = @i4
							
						end
				
						-- если не покрывается кусок 4 и 2 уровнями (онлайн и перерыв)
						-- или юзер в офлайне вообще в этом промежутке
						if (( @p4 = 0 ) and ( @p2 = 0 )) or ( @p5 = 0 )
 							set @dontinsert = 1
					
						
						-- разбор состояния на необходимость вставки			
						set @updatelast = 0
						if ( @lastt = 0 or @lastt is null ) begin
							-- если предыдущий был пустым блоком, или самое начало - обновляем величины
							set @updatelast = 1
							
						end else if ( @dontinsert = 0 ) and ( @actualst = @lastst ) begin
							-- если активный блок но статус совпадает с предыдущим блоком - обновляем время конца
							set @lastdstop = @dstop
							
						end	else begin
							-- в противном случае вставляем и обновляем величины
							if ( DateDiff ( ms, @lastdstart, @lastdstop ) > 10 ) 
								Insert Into @res
								Values ( @lastt, @lastid, @lastst, @lastpr, @lastdstart, @lastdstop )
						
							set @updatelast = 1
							
						end
					
						if ( @updatelast = 1 ) begin
							if ( @dontinsert = 1 ) begin
			  				
			  					set @lastt = 0
							
							end else begin
							
								set @lastt = @actualt
								set @lastid = @actualid
								set @lastst = @actualst
								set @lastpr = @actualpr
								set @lastdstart = @dstart
								set @lastdstop = @dstop
								
							end
						end
					
				
						--- оцениваем после вставки (конец) -----
						if ( @nk2 = 1 ) begin
							-- Новое (06.2010)
							--    Условие на неотрицательность

							if ( @pr2 = 0 ) begin
							
								if ( @p0 > 0 ) 
									set @p0 = @p0 - 1 
							
							end else if ( @pr2 = 1 ) begin
								
								if ( @p1 > 0 ) 
									set @p1 = @p1 - 1 
								
							end else if ( @pr2 = 2 ) begin
								
								if ( @p2 > 0 ) 
									set @p2 = @p2 - 1 
								
							end else if ( @pr2 = 3 ) begin
								
								if ( @p3 > 0 ) 
									set @p3 = @p3 - 1 
								
							end else if ( @pr2 = 4 ) begin
							
								if ( @p4 > 0 ) 
									set @p4 = @p4 - 1 
								
							end else if ( @pr2 = 5 ) begin
								
								if ( @p5 > 0 ) 
									set @p5 = @p5 - 1 
								
							end
						end
					
						Fetch Next 
							From tcursor 
							Into @o1, @o2, @st1, @pr1, @st2, @pr2, @dstart, @dstop, @nk1, @nk2, @tb1, @tb2, @id1, @id2
					End


					-- в конце последний промежуток надо втавить, так как алгоритм запаздывающий
					if ( @dontinsert = 0 )
						if ( DateDiff ( ms, @lastdstart, @lastdstop ) > 10 ) 
							Insert Into @res
							Values ( @lastt, @lastid, @lastst, @lastpr, @lastdstart, @lastdstop )

					Close tcursor
					Deallocate tcursor

					
					-------------------------------------------------------------------------------------------------
					-- обновляем данные по таблицам и формируем итоговую по оператору -------------------------------
					-------------------------------------------------------------------------------------------------

					--проверяем наличие записи об операторе в таблице (местами надо поменять с куском обновления и вставки в куб)
					Update [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
						Set p0 = @p0, p1 = @p1, p2 = @p2, p3 = @p3, p4 = @p4, p5 = @p5,
							s0 = @s0, s1 = @s1, s2 = @s2, s3 = @s3, s4 = @s4, s5 = @s5,
							LastDT = @dtnow
					Where Id = @idoperator

					Declare @operatorstates table
					(
						Id				int identity ( 1, 1 ) not null primary key,
						IdTask 			uniqueidentifier,
						IsOutput		bit,
						IdOperator 		uniqueidentifier,
						CallResult		int,
						State			int not null,
						DateTimeStart 	DateTime,
						DateStart 		DateTime,
						TimeStart 		DateTime,
						DateTimeStop 	DateTime,
						LenTime 		Float,
						ICode			int,
						IdEffort		uniqueidentifier
					)

					Delete From @operatorstates

					Insert into @operatorstates ( IdOperator, State, DateTimeStart, DateTimeStop, DateStart, TimeStart, LenTime, 
								ICode, IdTask, IsOutput, CallResult, IdEffort )
						Select @idoperator, r.State, r.DStart, r.DStop,
							Cast ( Floor ( Cast ( DStart as Float ) ) as DateTime ),
							Cast ( Cast ( DStart as float ) - Floor ( Cast ( DStart as Float ) ) as DateTime ),				
							Cast ( DStop - DStart as float ) * 86400,
							t2.ICode,
							IdTask, IsOutput, CallResult, IdEffort
						From @res r
							Left join @tempo1 t1 on t1.Id = r.Id and r.TableN = 1
							Left join @tempo2 t2 on t2.State = r.State and t2.DateTimeStart = r.DStart


					-- В Tempo2 лежат ICode, но так как состояние перерыва фактически будет по концу коммутации, если оная была
					--   а пересечение с Tempo2 по времени, то нужно обновить потом ICode там где не проставился, а в Tempo2 есть
					Update os
					Set os.ICode = t2.ICode
					From @operatorstates os, @tempo2 t2
					Where (( os.State = 9 ) and ( os.ICode is NULL ))
						and ( t2.State = 9)
						and ( t2.DateTimeStart < os.DateTimeStart and t2.DateTimeStop > os.DateTimeStart )


		
					-- Код крайней остающейся записи в кубе в настоящее время
					Declare @idt1 int
					Select @idt1 = MaxId
					From @maxid
					Where IdOperator = @idoperator

					-- Новое (06.2010)
					--    удаляем последние 10 минут
					-- i. IdOperator, DateTimeStart
					Delete From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates]
					Where IdOperator = @idoperator
						--And Id > @idt1
						And DateTimeStart >= @dts

		
					--   Обратно возвращаем на стык значение DateTimeStop
					if ( not @idt1 is null ) begin
					
						Update [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
						Set DateTimeStop = @dts,
							LenTime = Cast ( @dts - DateTimeStart as float ) * 86400
						Where Id = @idt1
							And DateTimeStop > @dts
							
					end


					--для склейки результатов выполнения задач (если расчет в середине выполнения - в первые состояния информация о результате не попала)
					Declare @cr table ( idtask uniqueidentifier, ideffort uniqueidentifier, callresult int )
					Insert into @cr ( idtask, ideffort, callresult )
						Select IdTask, IdEffort, CallResult
						From @operatorstates
						Group By IdTask, IdEffort, CallResult

					-- i. IdOperator, IdEffort, IdTask, CallResult
					Update os
					Set CallResult = cr.callresult
					From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] os, @cr cr
					Where os.IdOperator = @idoperator
						and os.IdEffort = cr.ideffort
						and not ( cr.IdEffort is null )
						and os.CallResult is null

					-- код склейки крайних записей
					Declare @state1 int, @dtstart1 DateTime, @dtstop1 DateTime
					
					if ( not @idt1 is null ) begin
					
						Select @state1 = State, @dtstart1 = DateTimeStart, @dtstop1 = DateTimeStop 
						From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( TabLock )
						Where Id = @idt1
						
						Declare @idt2 int, @state2 int, @dtstart2 DateTime, @dtstop2 DateTime
						Select Top 1 @idt2 = Id, @state2 = State, @dtstart2 = DateTimeStart, @dtstop2 = DateTimeStop
						From @operatorstates
						Order By DateTimeStart

						if ( @state1 = @state2 ) and ( @dtstart2 = @dtstop1 ) begin
						
							Update [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
							Set DateTimeStop = @dtstop2,
								LenTime = Cast ( @dtstop2 - @dtstart1 as float ) * 86400
							Where Id = @idt1

							Delete 
							From @operatorstates
							Where Id = @idt2
							
			   			end
					end

				
					-- обновление куба
					Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
							( IdTask, IsOutput, IdOperator, CallResult, State, DateTimeStart, DateStart, TimeStart, DateTimeStop, LenTime, ICode, IdEffort )
						Select IdTask, IsOutput, IdOperator, CallResult, State, DateTimeStart, DateStart, TimeStart, DateTimeStop, LenTime, ICode, IdEffort
						From @operatorstates
										
				end else begin
				
					Update [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
					Set LastDT = @dtnow
					Where Id = @idoperator
					
				end

			end
						
			Fetch Next 
				From ucursor 
				Into @idoperator, @nameoperator
		end

		Close ucursor
		Deallocate ucursor
		
		
		-- 14.04.2005
		-- Проблема рубленых состояний (дублированные более короткие
		Delete
		From [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates]
		Where DateTimeStart > DateAdd(hh, -3, GetDate()) 
			and exists(select Id from [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates] t2 
						where [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates].[DateTimeStop] = t2.[DateTimeStop] 
							and [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates].[DateTimeStart] > t2.[DateTimeStart]
							and [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates].[IdEffort]=t2.[IdEffort]
							and [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates].[State]=t2.[State]
							and [oktell_cc_temp].[dbo].[A_Cube_CC_operatorstates].[IdOperator]=t2.[IdOperator])


	Commit Tran TranUpdateOperatorStates
	
end
GO