﻿if not (object_id ( 'dbo.A_CallCenter_Temp_GetCollection' ) is null )
	Drop procedure dbo.A_CallCenter_Temp_GetCollection
GO

Create procedure [dbo].[A_CallCenter_Temp_GetCollection]
	@category int,
	@authorizeduserid uniqueidentifier
as
begin

	if ( @category = 5 ) begin /* OperatorState */

		If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
			If exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorStateTypes' ) )
				Select 1 as idtype, cat.Id, cat.Name 
				From oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorStateTypes cat With (NOLOCK)
				Order by cat.Name
	
	end else if ( @category = 4 ) begin /* TaskResult */
	
		If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
			If exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_TaskResultTypes' ) )
				Select 1 as idtype, cat.Id, cat.Name 
				From oktell_cc_temp.dbo.A_Cube_CC_Cat_TaskResultTypes cat With (NOLOCK)
				Order by cat.Name
	
	end else if ( @category = 3 ) begin /* Operator */
		
		If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
			If exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo' ) )		
				Select 0 as idtype, cat.Id, cat.Name 
				From oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo cat With (NOLOCK)
					Inner join dbo.A_Users u on u.Id = cat.Id
				Where (u.IsDeleted is null) or (u.IsDeleted = 0)
				Order by cat.Name
				
	end else if ( @category = 2 ) begin /* Project */
		
		If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
			If exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Project' ) ) begin
								
				if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privilegesP' ))
					Drop table #privilegesP
				
				Create table #privilegesP ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))	
				Insert into #privilegesP ( [idObject], [idPrivilege], [code] )
					exec A_Policy_SubjectPrivileges_GetForUserByObjectType
						@iduser = @authorizeduserid,
						@objecttype = 101 
						
				Select 0 as idtype, t.[Id], t.[Name]
				From (
						Select Distinct [Id]
						From oktell_cc_temp.[dbo].A_Cube_CC_Cat_Project t With (NOLOCK) 
							Inner join #privilegesP p on p.[idObject] = t.[Id]
					) a
					Inner join oktell_cc_temp.[dbo].A_Cube_CC_Cat_Project t With (NOLOCK) on t.[Id] = a.[Id]
				Order by t.[Name]
				
			end
			
	end else if ( @category = 1 ) begin /* Task */
	
		If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
			If exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Task' ) ) begin
							
				if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privilegesT' ))
					Drop table #privilegesT
				
				Create table #privilegesT ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))
				Insert into #privilegesT ( [idObject], [idPrivilege], [code] )
					exec A_Policy_SubjectPrivileges_GetForUserByObjectType
						@iduser = @authorizeduserid,
						@objecttype = 102 
						
				Select 0 as idtype, t.[Id], IsNull(proj.[Name] + ' - ', '') + t.[Name]
				From (
						Select Distinct [Id]
						From oktell_cc_temp.[dbo].A_Cube_CC_Cat_Task t With (NOLOCK)
							Inner join #privilegesT p on p.[idObject] = t.[Id]
					) a
					Inner join oktell_cc_temp.[dbo].A_Cube_CC_Cat_Task t With (NOLOCK) on t.[Id] = a.[Id] 
					Left join dbo.A_TaskManager_Tasks task on task.Id = t.Id
					Left join dbo.A_TaskManager_Projects proj on proj.Id = task.IdProject
				Order by proj.Name, t.[Name]
				
			end
			
	end
	
end
GO